@@ -1,6 +1,6 @@
-/* dvd.c
+/* bd.c
 
-   Copyright (c) 2003-2019 HandBrake Team
+   Copyright (c) 2003-2024 HandBrake Team
    This file is part of the HandBrake source code
    Homepage: <http://handbrake.fr/>.
    It may be used under the terms of the GNU General Public License v2.
@@ -88,7 +88,7 @@ int hb_bd_title_count( hb_bd_t * d )
     return d->title_count;
 }
 
-static void add_subtitle(int track, hb_list_t *list_subtitle, BLURAY_STREAM_INFO *bdsub, uint32_t codec)
+static void add_subtitle(int track, hb_list_t *list_subtitle, BLURAY_STREAM_INFO *bdsub, uint32_t codec, uint32_t codec_param)
 {
     hb_subtitle_t * subtitle;
     iso639_lang_t * lang;
@@ -111,7 +111,7 @@ static void add_subtitle(int track, hb_list_t *list_subtitle, BLURAY_STREAM_INFO
             return;
     }
     lang = lang_for_code2( (char*)bdsub->lang );
-    snprintf(subtitle->lang, sizeof( subtitle->lang ), "%s [%s]",
+    snprintf(subtitle->lang, sizeof( subtitle->lang ), "%s (%s)",
              strlen(lang->native_name) ? lang->native_name : lang->eng_name,
              hb_subsource_name(subtitle->source));
     snprintf(subtitle->iso639_2, sizeof( subtitle->iso639_2 ), "%s",
@@ -120,6 +120,7 @@ static void add_subtitle(int track, hb_list_t *list_subtitle, BLURAY_STREAM_INFO
     subtitle->reg_desc     = STR4_TO_UINT32("HDMV");
     subtitle->stream_type  = bdsub->coding_type;
     subtitle->codec        = codec;
+    subtitle->codec_param  = codec_param;
     subtitle->timebase.num = 1;
     subtitle->timebase.den = 90000;
 
@@ -130,7 +131,7 @@ static void add_subtitle(int track, hb_list_t *list_subtitle, BLURAY_STREAM_INFO
     return;
 }
 
-static void add_audio(int track, hb_list_t *list_audio, BLURAY_STREAM_INFO *bdaudio, int substream_type, uint32_t codec, uint32_t codec_param)
+static void add_audio(int index, int linked_index, int track, hb_list_t *list_audio, BLURAY_STREAM_INFO *bdaudio, int substream_type, uint32_t codec, uint32_t codec_param, int attributes)
 {
     const char * codec_name;
     hb_audio_t * audio;
@@ -139,6 +140,13 @@ static void add_audio(int track, hb_list_t *list_audio, BLURAY_STREAM_INFO *bdau
     audio = calloc( sizeof( hb_audio_t ), 1 );
 
     audio->id = (substream_type << 16) | bdaudio->pid;
+    audio->config.index = index;
+    if (linked_index >= 0)
+    {
+        audio->config.list_linked_index = hb_list_init();
+        hb_list_add_dup(audio->config.list_linked_index,
+                        &linked_index, sizeof(linked_index));
+    }
     audio->config.in.reg_desc = STR4_TO_UINT32("HDMV");
     audio->config.in.stream_type = bdaudio->coding_type;
     audio->config.in.substream_type = substream_type;
@@ -195,7 +203,7 @@ static void add_audio(int track, hb_list_t *list_audio, BLURAY_STREAM_INFO *bdau
 
     lang = lang_for_code2( (char*)bdaudio->lang );
 
-    audio->config.lang.attributes = HB_AUDIO_ATTR_NONE;
+    audio->config.lang.attributes = attributes;
 
     snprintf( audio->config.lang.simple,
               sizeof( audio->config.lang.simple ), "%s",
@@ -326,13 +334,6 @@ hb_title_t * hb_bd_title_scan( hb_bd_t * d, int tt, uint64_t min_duration )
 
     hb_log( "bd: playlist %05d.MPLS", ti->playlist );
     title->playlist = ti->playlist;
-
-    uint64_t pkt_count = 0;
-    for ( ii = 0; ii < ti->clip_count; ii++ )
-    {
-        pkt_count += ti->clips[ii].pkt_count;
-    }
-
     title->angle_count = ti->angle_count;
 
     /* Get duration */
@@ -466,55 +467,158 @@ hb_title_t * hb_bd_title_scan( hb_bd_t * d, int tt, uint64_t min_duration )
     for (ii = 0; ii < ti->clips[audio_clip_index].audio_stream_count; ii++)
     {
         BLURAY_STREAM_INFO * bdaudio;
+        int                  index;
 
         bdaudio = &ti->clips[audio_clip_index].audio_streams[ii];
+        index   = hb_list_count(title->list_audio);
 
         switch (bdaudio->coding_type)
         {
             case BLURAY_STREAM_TYPE_AUDIO_TRUHD:
                 // Add 2 audio tracks.  One for TrueHD and one for AC-3
-                add_audio(ii, title->list_audio, bdaudio, HB_SUBSTREAM_BD_AC3,
-                          HB_ACODEC_AC3, AV_CODEC_ID_AC3);
-                add_audio(ii, title->list_audio, bdaudio, HB_SUBSTREAM_BD_TRUEHD,
-                          HB_ACODEC_FFTRUEHD, AV_CODEC_ID_TRUEHD);
+                add_audio(index, index + 1, ii, title->list_audio, bdaudio, HB_SUBSTREAM_BD_AC3,
+                          HB_ACODEC_AC3, AV_CODEC_ID_AC3, HB_AUDIO_ATTR_NONE);
+                add_audio(index + 1, index, ii, title->list_audio, bdaudio, HB_SUBSTREAM_BD_TRUEHD,
+                          HB_ACODEC_FFTRUEHD, AV_CODEC_ID_TRUEHD,
+                          HB_AUDIO_ATTR_NONE);
                 break;
 
             case BLURAY_STREAM_TYPE_AUDIO_DTS:
-                add_audio(ii, title->list_audio, bdaudio, 0,
-                          HB_ACODEC_DCA, AV_CODEC_ID_DTS);
+                add_audio(index, -1, ii, title->list_audio, bdaudio, 0,
+                          HB_ACODEC_DCA, AV_CODEC_ID_DTS, HB_AUDIO_ATTR_NONE);
                 break;
 
             case BLURAY_STREAM_TYPE_AUDIO_MPEG2:
             case BLURAY_STREAM_TYPE_AUDIO_MPEG1:
-                add_audio(ii, title->list_audio, bdaudio, 0,
-                          HB_ACODEC_FFMPEG, AV_CODEC_ID_MP2);
+                add_audio(index, -1, ii, title->list_audio, bdaudio, 0,
+                          HB_ACODEC_FFMPEG, AV_CODEC_ID_MP2,
+                          HB_AUDIO_ATTR_NONE);
                 break;
 
             case BLURAY_STREAM_TYPE_AUDIO_AC3PLUS:
-                add_audio(ii, title->list_audio, bdaudio, 0,
-                          HB_ACODEC_FFEAC3, AV_CODEC_ID_EAC3);
+                add_audio(index, -1, ii, title->list_audio, bdaudio, 0,
+                          HB_ACODEC_FFEAC3, AV_CODEC_ID_EAC3,
+                          HB_AUDIO_ATTR_NONE);
+                break;
+
+            case BLURAY_STREAM_TYPE_AUDIO_AC3PLUS_SECONDARY:
+                add_audio(index, -1, ii, title->list_audio, bdaudio, 0,
+                          HB_ACODEC_FFEAC3, AV_CODEC_ID_EAC3,
+                          HB_AUDIO_ATTR_NONE);
                 break;
 
             case BLURAY_STREAM_TYPE_AUDIO_LPCM:
-                add_audio(ii, title->list_audio, bdaudio, 0,
-                          HB_ACODEC_FFMPEG, AV_CODEC_ID_PCM_BLURAY);
+                add_audio(index, -1, ii, title->list_audio, bdaudio, 0,
+                          HB_ACODEC_FFMPEG, AV_CODEC_ID_PCM_BLURAY,
+                          HB_AUDIO_ATTR_NONE);
                 break;
 
             case BLURAY_STREAM_TYPE_AUDIO_AC3:
-                add_audio(ii, title->list_audio, bdaudio, 0,
-                          HB_ACODEC_AC3, AV_CODEC_ID_AC3);
+                add_audio(index, -1, ii, title->list_audio, bdaudio, 0,
+                          HB_ACODEC_AC3, AV_CODEC_ID_AC3, HB_AUDIO_ATTR_NONE);
                 break;
 
             case BLURAY_STREAM_TYPE_AUDIO_DTSHD_MASTER:
             case BLURAY_STREAM_TYPE_AUDIO_DTSHD:
                 // Add 2 audio tracks.  One for DTS-HD and one for DTS
-                add_audio(ii, title->list_audio, bdaudio, HB_SUBSTREAM_BD_DTS,
-                          HB_ACODEC_DCA, AV_CODEC_ID_DTS);
-                // DTS-HD is special.  The substreams must be concatinated
+                add_audio(index, index + 1, ii, title->list_audio, bdaudio, HB_SUBSTREAM_BD_DTS,
+                          HB_ACODEC_DCA, AV_CODEC_ID_DTS, HB_AUDIO_ATTR_NONE);
+                // DTS-HD is special.  The substreams must be concatenated
                 // DTS-core followed by DTS-hd-extensions.  Setting
                 // a substream id of 0 says use all substreams.
-                add_audio(ii, title->list_audio, bdaudio, 0,
-                          HB_ACODEC_DCA_HD, AV_CODEC_ID_DTS);
+                add_audio(index + 1, index, ii, title->list_audio, bdaudio, 0,
+                          HB_ACODEC_DCA_HD, AV_CODEC_ID_DTS,
+                          HB_AUDIO_ATTR_NONE);
+                break;
+
+            case BLURAY_STREAM_TYPE_AUDIO_DTSHD_SECONDARY:
+                // BD "DTSHD_SECONDARY" is DTS Express which has no
+                // DTS core
+                add_audio(index, -1, ii, title->list_audio, bdaudio, 0,
+                          HB_ACODEC_DCA_HD, AV_CODEC_ID_DTS,
+                          HB_AUDIO_ATTR_NONE);
+                break;
+
+            default:
+                hb_log("scan: unknown audio pid 0x%x codec 0x%x", bdaudio->pid,
+                       bdaudio->coding_type);
+                break;
+        }
+    }
+
+    // Add all the secondary audios found in the above clip.
+    for (jj = 0; jj < ti->clips[audio_clip_index].sec_audio_stream_count; jj++, ii++)
+    {
+        BLURAY_STREAM_INFO * bdaudio;
+        int                  index;
+
+        bdaudio = &ti->clips[audio_clip_index].sec_audio_streams[jj];
+        index   = hb_list_count(title->list_audio);
+
+        switch (bdaudio->coding_type)
+        {
+            case BLURAY_STREAM_TYPE_AUDIO_TRUHD:
+                // Add 2 audio tracks.  One for TrueHD and one for AC-3
+                add_audio(index, index + 1, ii, title->list_audio, bdaudio, HB_SUBSTREAM_BD_AC3,
+                          HB_ACODEC_AC3, AV_CODEC_ID_AC3,
+                          HB_AUDIO_ATTR_SECONDARY);
+                add_audio(index + 1, index, ii, title->list_audio, bdaudio, HB_SUBSTREAM_BD_TRUEHD,
+                          HB_ACODEC_FFTRUEHD, AV_CODEC_ID_TRUEHD,
+                          HB_AUDIO_ATTR_SECONDARY);
+                break;
+
+            case BLURAY_STREAM_TYPE_AUDIO_DTS:
+                add_audio(index, -1, ii, title->list_audio, bdaudio, 0,
+                          HB_ACODEC_DCA, AV_CODEC_ID_DTS,
+                          HB_AUDIO_ATTR_SECONDARY);
+                break;
+
+            case BLURAY_STREAM_TYPE_AUDIO_MPEG2:
+            case BLURAY_STREAM_TYPE_AUDIO_MPEG1:
+                add_audio(index, -1, ii, title->list_audio, bdaudio, 0,
+                          HB_ACODEC_FFMPEG, AV_CODEC_ID_MP2,
+                          HB_AUDIO_ATTR_SECONDARY);
+                break;
+
+            case BLURAY_STREAM_TYPE_AUDIO_AC3PLUS:
+            case BLURAY_STREAM_TYPE_AUDIO_AC3PLUS_SECONDARY:
+                add_audio(index, -1, ii, title->list_audio, bdaudio, 0,
+                          HB_ACODEC_FFEAC3, AV_CODEC_ID_EAC3,
+                          HB_AUDIO_ATTR_SECONDARY);
+                break;
+
+            case BLURAY_STREAM_TYPE_AUDIO_LPCM:
+                add_audio(index, -1, ii, title->list_audio, bdaudio, 0,
+                          HB_ACODEC_FFMPEG, AV_CODEC_ID_PCM_BLURAY,
+                          HB_AUDIO_ATTR_SECONDARY);
+                break;
+
+            case BLURAY_STREAM_TYPE_AUDIO_AC3:
+                add_audio(index, -1, ii, title->list_audio, bdaudio, 0,
+                          HB_ACODEC_AC3, AV_CODEC_ID_AC3,
+                          HB_AUDIO_ATTR_SECONDARY);
+                break;
+
+            case BLURAY_STREAM_TYPE_AUDIO_DTSHD_MASTER:
+            case BLURAY_STREAM_TYPE_AUDIO_DTSHD:
+                // Add 2 audio tracks.  One for DTS-HD and one for DTS
+                add_audio(index, index + 1, ii, title->list_audio, bdaudio, HB_SUBSTREAM_BD_DTS,
+                          HB_ACODEC_DCA, AV_CODEC_ID_DTS,
+                          HB_AUDIO_ATTR_SECONDARY);
+                // DTS-HD is special.  The substreams must be concatenated
+                // DTS-core followed by DTS-hd-extensions.  Setting
+                // a substream id of 0 says use all substreams.
+                add_audio(index + 1, index, ii, title->list_audio, bdaudio, 0,
+                          HB_ACODEC_DCA_HD, AV_CODEC_ID_DTS,
+                          HB_AUDIO_ATTR_SECONDARY);
+                break;
+
+            case BLURAY_STREAM_TYPE_AUDIO_DTSHD_SECONDARY:
+                // BD "DTSHD_SECONDARY" is DTS Express which has no
+                // DTS core
+                add_audio(index, -1, ii, title->list_audio, bdaudio, 0,
+                          HB_ACODEC_DCA_HD, AV_CODEC_ID_DTS,
+                          HB_AUDIO_ATTR_SECONDARY);
                 break;
 
             default:
@@ -534,7 +638,8 @@ hb_title_t * hb_bd_title_scan( hb_bd_t * d, int tt, uint64_t min_duration )
         switch( bdpgs->coding_type )
         {
             case BLURAY_STREAM_TYPE_SUB_PG:
-                add_subtitle(ii, title->list_subtitle, bdpgs, WORK_DECPGSSUB);
+                add_subtitle(ii, title->list_subtitle, bdpgs, WORK_DECAVSUB,
+                             AV_CODEC_ID_HDMV_PGS_SUBTITLE);
                 break;
             default:
                 hb_log( "scan: unknown subtitle pid 0x%x codec 0x%x",
@@ -549,10 +654,11 @@ hb_title_t * hb_bd_title_scan( hb_bd_t * d, int tt, uint64_t min_duration )
         char chapter_title[80];
 
         // Sanity check start time of this chapter.
-        // If it is beyond the end of the title, drop it.
-        if (ti->chapters[ii].start > ti->duration)
+        // If the chapter starts within 1.5 seconds of the end of
+        // the title, drop it.
+        if (ti->duration - ti->chapters[ii].start < 90000 * 1.5)
         {
-            hb_log("bd: chapter %d invalid start %"PRIu64", dropping", ii+1,
+            hb_log("bd: chapter %d too short %"PRIu64", dropping", ii+1,
                    ti->chapters[ii].start);
             continue;
         }
@@ -560,7 +666,7 @@ hb_title_t * hb_bd_title_scan( hb_bd_t * d, int tt, uint64_t min_duration )
         chapter = calloc( sizeof( hb_chapter_t ), 1 );
 
         chapter->index = ++jj;
-        sprintf( chapter_title, "Chapter %d", chapter->index );
+        snprintf( chapter_title, sizeof(chapter_title), "Chapter %d", chapter->index );
         hb_chapter_set_title( chapter, chapter_title );
 
         chapter->duration = ti->chapters[ii].duration;
@@ -607,7 +713,7 @@ hb_title_t * hb_bd_title_scan( hb_bd_t * d, int tt, uint64_t min_duration )
 
         hb_list_add( title->list_chapter, chapter );
     }
-    hb_log( "bd: title %d has %d chapters", tt, ti->chapter_count );
+    hb_log( "bd: title %d has %d chapters", tt, hb_list_count(title->list_chapter));
 
     /* This title is ok so far */
     goto cleanup;
@@ -628,9 +734,9 @@ int hb_bd_main_feature( hb_bd_t * d, hb_list_t * list_title )
     int longest = 0;
     int ii;
     uint64_t longest_duration = 0;
-    int highest_rank = 0;
+    int highest_rank = 0, rank;
     int most_chapters = 0;
-    int rank[8] = {0, 1, 3, 2, 6, 5, 7, 4};
+    int ranks[9] = {0, 1, 3, 2, 6, 5, 7, 4, 8};
     BLURAY_TITLE_INFO * ti;
 
     for ( ii = 0; ii < hb_list_count( list_title ); ii++ )
@@ -642,16 +748,21 @@ int hb_bd_main_feature( hb_bd_t * d, hb_list_t * list_title )
             BLURAY_STREAM_INFO * bdvideo = &ti->clips[0].video_streams[0];
             if ( title->duration > longest_duration * 0.7 && bdvideo->format < 8 )
             {
-                if (highest_rank < rank[bdvideo->format] ||
+                rank = 0;
+                if (bdvideo->format <= 8)
+                {
+                    rank = ranks[bdvideo->format];
+                }
+                if (highest_rank < rank ||
                     ( title->duration > longest_duration &&
-                          highest_rank == rank[bdvideo->format]))
+                      highest_rank == rank))
                 {
                     longest = title->index;
                     longest_duration = title->duration;
-                    highest_rank = rank[bdvideo->format];
+                    highest_rank = rank;
                     most_chapters = ti->chapter_count;
                 }
-                else if (highest_rank == rank[bdvideo->format] &&
+                else if (highest_rank == rank &&
                          title->duration == longest_duration &&
                          ti->chapter_count > most_chapters)
                 {
@@ -813,6 +924,8 @@ hb_buffer_t * hb_bd_read( hb_bd_t * d )
             {
                 // A unit is 6144 bytes (32 TS packets).  Give up after we've
                 // seen > 6MB of invalid data.
+                hb_error("bd: Error, too many consecutive bad units.");
+                hb_set_work_error(d->h, HB_ERROR_READ);
                 return NULL;
             }
             continue;
@@ -840,7 +953,6 @@ hb_buffer_t * hb_bd_read( hb_bd_t * d )
             return out;
         }
     }
-    return NULL;
 }
 
 /***********************************************************************
