@@ -1,37 +1,31 @@
 /* ports.c
 
-   Copyright (c) 2003-2018 HandBrake Team
+   Copyright (c) 2003-2024 HandBrake Team
    This file is part of the HandBrake source code
    Homepage: <http://handbrake.fr/>.
    It may be used under the terms of the GNU General Public License v2.
    For full terms see the file COPYING file or visit http://www.gnu.org/licenses/gpl-2.0.html
  */
 
+#include "handbrake/project.h"
+
 #ifdef SYS_MINGW
-#define _WIN32_WINNT 0x600
+#define _WIN32_WINNT 0x0601
 #endif
 
-#ifdef USE_PTHREAD
 #ifdef SYS_LINUX
 #define _GNU_SOURCE
 #include <sched.h>
 #endif
 #include <pthread.h>
-#endif
 
-#ifdef SYS_BEOS
-#include <kernel/OS.h>
-#endif
-
-#if defined(SYS_DARWIN) || defined(SYS_FREEBSD)
+#if defined(SYS_DARWIN) || defined(SYS_FREEBSD) || defined(SYS_NETBSD) || defined(SYS_OPENBSD)
 #include <sys/types.h>
 #include <sys/sysctl.h>
+#if HB_PROJECT_FEATURE_QSV && defined(SYS_FREEBSD)
+#include <libdrm/drm.h>
+#include <fcntl.h>
 #endif
-
-#ifdef SYS_OPENBSD
-#include <sys/param.h>
-#include <sys/sysctl.h>
-#include <machine/cpu.h>
 #endif
 
 #ifdef SYS_MINGW
@@ -70,10 +64,9 @@
 #include <linux/cdrom.h>
 #include <fcntl.h>
 #include <sys/ioctl.h>
-#elif defined( SYS_OPENBSD )
-#include <sys/dvdio.h>
-#include <fcntl.h>
-#include <sys/ioctl.h>
+#if HB_PROJECT_FEATURE_QSV
+#include <libdrm/drm.h>
+#endif
 #endif
 
 #ifdef __APPLE__
@@ -83,7 +76,7 @@
 #include <stddef.h>
 #include <unistd.h>
 
-#include "hb.h"
+#include "handbrake/handbrake.h"
 #include "libavutil/cpu.h"
 
 /************************************************************************
@@ -110,35 +103,7 @@ int gettimeofday( struct timeval * tv, struct timezone * tz )
 #endif
 */
 
-// Convert utf8 string to current code page.
-// The internal string representation in hb is utf8. But some
-// libraries (libmkv, and mp4v2) expect filenames in the current
-// code page.  So we must convert.
-char * hb_utf8_to_cp(const char *src)
-{
-    char *dst = NULL;
-
-#if defined( SYS_MINGW )
-    int num_chars = MultiByteToWideChar(CP_UTF8, 0, src, -1, NULL, 0);
-    if (num_chars <= 0)
-        return NULL;
-    wchar_t * tmp = calloc(num_chars, sizeof(wchar_t));
-    MultiByteToWideChar(CP_UTF8, 0, src, -1, tmp, num_chars);
-    int len = WideCharToMultiByte(GetACP(), 0, tmp, num_chars, NULL, 0, NULL, NULL);
-    if (len <= 0)
-        return NULL;
-    dst = calloc(len, sizeof(char));
-    WideCharToMultiByte(GetACP(), 0, tmp, num_chars, dst, len, NULL, NULL);
-    free(tmp);
-#else
-    // Other platforms don't have code pages
-    dst = strdup(src);
-#endif
-
-    return dst;
-}
-
-int hb_dvd_region(char *device, int *region_mask)
+int hb_dvd_region(const char *device, int *region_mask)
 {
 #if defined( DVD_LU_SEND_RPC_STATE ) && defined( DVD_AUTH )
     struct stat  st;
@@ -211,12 +176,10 @@ void hb_snooze( int delay )
     {
         return;
     }
-#if defined( SYS_BEOS )
-    snooze( 1000 * delay );
-#elif defined( SYS_DARWIN ) || defined( SYS_LINUX ) || defined( SYS_FREEBSD) || defined( SYS_SunOS )
-    usleep( 1000 * delay );
-#elif defined( SYS_CYGWIN ) || defined( SYS_MINGW )
+#if defined( SYS_CYGWIN ) || defined( SYS_MINGW )
     Sleep( delay );
+#else
+    usleep( 1000 * delay );
 #endif
 }
 
@@ -239,25 +202,27 @@ struct
 
 int hb_get_cpu_count()
 {
+    init_cpu_info();
     return hb_cpu_info.count;
 }
 
 int hb_get_cpu_platform()
 {
+    init_cpu_info();
     return hb_cpu_info.platform;
 }
 
 const char* hb_get_cpu_name()
 {
+    init_cpu_info();
     return hb_cpu_info.name;
 }
 
 const char* hb_get_cpu_platform_name()
 {
+    init_cpu_info();
     switch (hb_cpu_info.platform)
     {
-        // Intel 64 and IA-32 Architectures Software Developer's Manual, Vol. 3C
-        // Table 35-1: CPUID Signature Values of DisplayFamily_DisplayModel
         case HB_CPU_PLATFORM_INTEL_BNL:
             return "Intel microarchitecture Bonnell";
         case HB_CPU_PLATFORM_INTEL_SNB:
@@ -276,6 +241,14 @@ const char* hb_get_cpu_platform_name()
             return "Intel microarchitecture Airmont";
         case HB_CPU_PLATFORM_INTEL_KBL:
             return "Intel microarchitecture Kaby Lake";
+        case HB_CPU_PLATFORM_INTEL_CML:
+            return "Intel microarchitecture Comet Lake";
+        case HB_CPU_PLATFORM_INTEL_ICL:
+            return "Intel microarchitecture Ice Lake";
+        case HB_CPU_PLATFORM_INTEL_TGL:
+            return "Intel microarchitecture Tiger Lake";
+        case HB_CPU_PLATFORM_INTEL_ADL:
+            return "Intel microarchitecture Alder Lake performance hybrid architecture";
         default:
             return NULL;
     }
@@ -301,21 +274,24 @@ const char* hb_get_cpu_platform_name()
 
 static void init_cpu_info()
 {
-    hb_cpu_info.name     = NULL;
+    if (hb_cpu_info.count != 0)
+        return;
+
+    hb_cpu_info.name     = "Unknown";
     hb_cpu_info.count    = init_cpu_count();
     hb_cpu_info.platform = HB_CPU_PLATFORM_UNSPECIFIED;
 
+#if ARCH_X86_64 || ARCH_X86_32
     if (av_get_cpu_flags() & AV_CPU_FLAG_SSE)
     {
-#if ARCH_X86_64 || ARCH_X86_32
         int eax, ebx, ecx, edx, family, model;
 
         cpuid(1, &eax, &ebx, &ecx, &edx);
         family = ((eax >> 8) & 0xf) + ((eax >> 20) & 0xff);
         model  = ((eax >> 4) & 0xf) + ((eax >> 12) & 0xf0);
 
-        // Intel 64 and IA-32 Architectures Software Developer's Manual, Vol. 3C
-        // Table 35-1: CPUID Signature Values of DisplayFamily_DisplayModel
+        // Intel 64 and IA-32 Architectures Software Developer's Manual, Volume 4/April 2022
+        // Table 2-1. CPUID Signature Values of DisplayFamily_DisplayModel
         switch (family)
         {
             case 0x06:
@@ -366,6 +342,22 @@ static void init_cpu_info()
                     case 0x9E:
                         hb_cpu_info.platform = HB_CPU_PLATFORM_INTEL_KBL;
                         break;
+                    case 0xA5:
+                    case 0xA6:
+                        hb_cpu_info.platform = HB_CPU_PLATFORM_INTEL_CML;
+                        break;
+                    case 0x7D:
+                    case 0x7E:
+                        hb_cpu_info.platform = HB_CPU_PLATFORM_INTEL_ICL;
+                        break;
+                    case 0x8C:
+                    case 0x8D:
+                        hb_cpu_info.platform = HB_CPU_PLATFORM_INTEL_TGL;
+                        break;
+                    case 0x97:
+                    case 0x9A:
+                        hb_cpu_info.platform = HB_CPU_PLATFORM_INTEL_ADL;
+                        break;
                     default:
                         break;
                 }
@@ -397,16 +389,29 @@ static void init_cpu_info()
                   &hb_cpu_info.buf4[10],
                   &hb_cpu_info.buf4[11]);
 
-            hb_cpu_info.name    = hb_cpu_info.buf;
-            hb_cpu_info.buf[47] = '\0'; // just in case
-
-            while (isspace(*hb_cpu_info.name))
-            {
-                // skip leading whitespace to prettify
-                hb_cpu_info.name++;
-            }
+            hb_cpu_info.name = hb_cpu_info.buf;
         }
-#endif // ARCH_X86_64 || ARCH_X86_32
+    }
+#elif defined(SYS_DARWIN)
+    size_t buflen = sizeof(hb_cpu_info.buf);
+    if (sysctlbyname("machdep.cpu.brand_string", &hb_cpu_info.buf, &buflen, NULL, 0) == 0)
+    {
+        hb_cpu_info.name = hb_cpu_info.buf;
+    }
+#endif
+
+    // ensure string is null-terminated and trim trailing whitespace
+    int ii = sizeof(hb_cpu_info.buf) - 1;
+    do {
+        hb_cpu_info.buf[ii] = '\0';
+        ii -= 1;
+    }
+    while (ii > 0 && isspace(hb_cpu_info.buf[ii]));
+
+    while (isspace(*hb_cpu_info.name))
+    {
+        // skip leading whitespace to prettify
+        hb_cpu_info.name++;
     }
 }
 
@@ -419,9 +424,7 @@ static int init_cpu_count()
     int cpu_count = 1;
 
 #if defined(SYS_CYGWIN) || defined(SYS_MINGW)
-    SYSTEM_INFO cpuinfo;
-    GetSystemInfo( &cpuinfo );
-    cpu_count = cpuinfo.dwNumberOfProcessors;
+    cpu_count = GetActiveProcessorCount(ALL_PROCESSOR_GROUPS);
 
 #elif defined(SYS_LINUX)
     unsigned int bit;
@@ -431,15 +434,14 @@ static int init_cpu_count()
     for( cpu_count = 0, bit = 0; bit < sizeof(p_aff); bit++ )
          cpu_count += (((uint8_t *)&p_aff)[bit / 8] >> (bit % 8)) & 1;
 
-#elif defined(SYS_BEOS)
-    system_info info;
-    get_system_info( &info );
-    cpu_count = info.cpu_count;
-
-#elif defined(SYS_DARWIN) || defined(SYS_FREEBSD) || defined(SYS_OPENBSD)
+#elif defined(SYS_DARWIN) || defined(SYS_FREEBSD) || defined(SYS_NETBSD) || defined(SYS_OPENBSD)
     size_t length = sizeof( cpu_count );
-#ifdef SYS_OPENBSD
+#if defined SYS_OPENBSD || defined(SYS_NETBSD)
+#ifdef HW_NCPUONLINE
+    int mib[2] = { CTL_HW, HW_NCPUONLINE };
+#else
     int mib[2] = { CTL_HW, HW_NCPU };
+#endif
     if( sysctl(mib, 2, &cpu_count, &length, NULL, 0) )
 #else
     if( sysctlbyname("hw.ncpu", &cpu_count, &length, NULL, 0) )
@@ -566,7 +568,7 @@ void hb_get_user_config_directory( char path[512] )
         return;
     }
 #elif defined( __APPLE__ )
-    if (osx_get_user_config_directory(path) == 0)
+    if (macOS_get_user_config_directory(path) == 0)
     {
         return;
     }
@@ -596,57 +598,81 @@ void hb_get_user_config_filename( char name[1024], char *fmt, ... )
 }
 
 /************************************************************************
- * Get a temporary directory for HB
+ * Creates a uniquely-named temporary directory for this process. On
+ * POSIX systems, mkdtemp(3) is used to ensure the directory name is
+ * unique even if multiple HandBrake instances have the same PID.
  ***********************************************************************/
-char * hb_get_temporary_directory()
+static pthread_once_t tmp_control = PTHREAD_ONCE_INIT;
+static char *tmp_dirname = NULL;
+
+static void
+hb_init_temporary_directory (void)
 {
-    char * path, * base, * p;
+    char *path, *base, *p;
 
-    /* Create the base */
 #if defined( SYS_CYGWIN ) || defined( SYS_MINGW )
     base = malloc(MAX_PATH);
-    int i_size = GetTempPath( MAX_PATH, base );
-    if( i_size <= 0 || i_size >= MAX_PATH )
+    int i_size = GetTempPath(MAX_PATH, base);
+    if (i_size <= 0 || i_size >= MAX_PATH)
     {
-        if( getcwd( base, MAX_PATH ) == NULL )
-            strcpy( base, "c:" ); /* Bad fallback but ... */
+        if (getcwd(base, MAX_PATH) == NULL)
+            strcpy(base, "c:"); /* Bad fallback but ... */
     }
 
     /* c:/path/ works like a charm under cygwin(win32?) so use it */
-    while( ( p = strchr( base, '\\' ) ) )
+    while ((p = strchr(base, '\\')))
         *p = '/';
+
+    /* I prefer to remove eventual last '/' (for cygwin) */
+    if (base[strlen(base)-1] == '/')
+        base[strlen(base)-1] = '\0';
+
+    path = hb_strdup_printf("%s/HandBrake-%d", base, (int)getpid());
+    hb_mkdir(path);
+    free(base);
 #else
-    if( (p = getenv( "TMPDIR" ) ) != NULL ||
-        (p = getenv( "TEMP" ) )   != NULL )
+    if ((p = getenv("TMPDIR")) != NULL ||
+        (p = getenv("TEMP"))   != NULL)
         base = strdup(p);
     else
         base = strdup("/tmp");
-#endif
-    /* I prefer to remove evntual last '/' (for cygwin) */
-    if( base[strlen(base)-1] == '/' )
+
+    if (base[strlen(base)-1] == '/')
         base[strlen(base)-1] = '\0';
 
-    path = hb_strdup_printf("%s/hb.%d", base, (int)getpid());
+    /* Create a new randomly-named directory from the template */
+    path = hb_strdup_printf("%s/handbrake-XXXXXX", base);
+    if (!mkdtemp(path))
+    {
+        /* We still use the path even if directory creation fails */
+        hb_error("Failed to create a temporary directory at %s\n", path);
+    }
     free(base);
+#endif
+    tmp_dirname = path;
+}
 
-    return path;
+const char *
+hb_get_temporary_directory (void)
+{
+    /* Ensure the directory name is only created once */
+    pthread_once(&tmp_control, hb_init_temporary_directory);
+    return tmp_dirname;
 }
 
 /************************************************************************
- * Get a tempory filename for HB
+ * Get a temporary filename for HB
  ***********************************************************************/
 char * hb_get_temporary_filename( char *fmt, ... )
 {
     va_list   args;
     char    * name, * path;
-    char    * dir = hb_get_temporary_directory();
 
     va_start( args, fmt );
     name = hb_strdup_vaprintf(fmt, args);
     va_end( args );
 
-    path = hb_strdup_printf("%s/%s", dir, name);
-    free(dir);
+    path = hb_strdup_printf("%s/%s", hb_get_temporary_directory(), name);
     free(name);
 
     return path;
@@ -778,7 +804,7 @@ char * hb_strr_dir_sep(const char *path)
  * Wrapper to the real mkdir, needed only because it doesn't take a
  * second argument on Win32. Grrr.
  ***********************************************************************/
-int hb_mkdir(char * path)
+int hb_mkdir(const char * path)
 {
 #ifdef SYS_MINGW
     wchar_t path_utf16[MAX_PATH];
@@ -802,14 +828,7 @@ struct hb_thread_s
 
     hb_lock_t     * lock;
     int             exited;
-
-#if defined( SYS_BEOS )
-    thread_id       thread;
-#elif USE_PTHREAD
     pthread_t       thread;
-//#elif defined( SYS_CYGWIN )
-//    HANDLE          thread;
-#endif
 };
 
 /* Get a unique identifier to thread and represent as 64-bit unsigned.
@@ -818,22 +837,18 @@ struct hb_thread_s
  */
 static uint64_t hb_thread_to_integer( const hb_thread_t* t )
 {
-#if defined( USE_PTHREAD )
-    #if defined( SYS_CYGWIN )
-        return (uint64_t)t->thread;
-    #elif defined( _WIN32 ) || defined( __MINGW32__ )
-    #if defined(PTW32_VERSION)
-        return (uint64_t)(ptrdiff_t)t->thread.p;
-    #else
-        return (uint64_t)t->thread;
-    #endif
-    #elif defined( SYS_DARWIN )
-        return (unsigned long)t->thread;
-    #else
-        return (uint64_t)t->thread;
-    #endif
+#if defined( SYS_CYGWIN )
+    return (uint64_t)t->thread;
+#elif defined( _WIN32 ) || defined( __MINGW32__ )
+# if defined(PTW32_VERSION)
+    return (uint64_t)(ptrdiff_t)t->thread.p;
+# else
+    return (uint64_t)t->thread;
+# endif
+#elif defined( SYS_DARWIN )
+    return (unsigned long)t->thread;
 #else
-    return 0;
+    return (uint64_t)t->thread;
 #endif
 }
 
@@ -841,7 +856,7 @@ static uint64_t hb_thread_to_integer( const hb_thread_t* t )
  * hb_thread_func()
  ************************************************************************
  * We use it as the root routine for any thread, for two reasons:
- *  + To set the thread priority on OS X (pthread_setschedparam() could
+ *  + To set the thread priority on macOS (pthread_setschedparam() could
  *    be called from hb_thread_init(), but it's nicer to do it as we
  *    are sure it is done before the real routine starts)
  *  + Get informed when the thread exits, so we know whether
@@ -851,16 +866,18 @@ static void attribute_align_thread hb_thread_func( void * _t )
 {
     hb_thread_t * t = (hb_thread_t *) _t;
 
-#if defined( SYS_DARWIN ) || defined( SYS_FREEBSD ) || defined ( __FreeBSD__ )
-    /* Set the thread priority */
+#if (defined( SYS_DARWIN ) && !defined(__aarch64__)) || defined( SYS_FREEBSD ) || \
+    defined( SYS_NETBSD ) || defined( SYS_OPENBSD )
+    /* Set the thread priority
+       Do not change priority on Darwin arm systems */
     struct sched_param param;
     memset( &param, 0, sizeof( struct sched_param ) );
     param.sched_priority = t->priority;
     pthread_setschedparam( pthread_self(), SCHED_OTHER, &param );
 #endif
 
-#if defined( SYS_BEOS )
-    signal( SIGINT, SIG_IGN );
+#if defined( SYS_DARWIN )
+    pthread_setname_np( t->name );
 #endif
 
     /* Start the actual routine */
@@ -894,24 +911,9 @@ hb_thread_t * hb_thread_init( const char * name, void (* function)(void *),
     t->lock     = hb_lock_init();
 
     /* Create and start the thread */
-#if defined( SYS_BEOS )
-    t->thread = spawn_thread( (thread_func) hb_thread_func,
-                              name, priority, t );
-    resume_thread( t->thread );
-
-#elif USE_PTHREAD
     pthread_create( &t->thread, NULL,
                     (void * (*)( void * )) hb_thread_func, t );
 
-//#elif defined( SYS_CYGWIN )
-//    t->thread = CreateThread( NULL, 0,
-//        (LPTHREAD_START_ROUTINE) hb_thread_func, t, 0, NULL );
-//
-//    /* Maybe use THREAD_PRIORITY_LOWEST instead */
-//    if( priority == HB_LOW_PRIORITY )
-//        SetThreadPriority( GetCurrentThread(), THREAD_PRIORITY_BELOW_NORMAL );
-#endif
-
     hb_deep_log( 2, "thread %"PRIx64" started (\"%s\")", hb_thread_to_integer( t ), t->name );
     return t;
 }
@@ -926,17 +928,8 @@ void hb_thread_close( hb_thread_t ** _t )
     hb_thread_t * t = *_t;
 
     /* Join the thread */
-#if defined( SYS_BEOS )
-    long exit_value;
-    wait_for_thread( t->thread, &exit_value );
-
-#elif USE_PTHREAD
     pthread_join( t->thread, NULL );
 
-//#elif defined( SYS_CYGWIN )
-//    WaitForSingleObject( t->thread, INFINITE );
-#endif
-
     hb_deep_log( 2, "thread %"PRIx64" joined (\"%s\")", hb_thread_to_integer( t ), t->name );
 
     hb_lock_close( &t->lock );
@@ -966,13 +959,7 @@ int hb_thread_has_exited( hb_thread_t * t )
  ***********************************************************************/
 struct hb_lock_s
 {
-#if defined( SYS_BEOS )
-    sem_id          sem;
-#elif USE_PTHREAD
     pthread_mutex_t mutex;
-//#elif defined( SYS_CYGWIN )
-//    HANDLE          mutex;
-#endif
 };
 
 /************************************************************************
@@ -987,21 +974,16 @@ hb_lock_t * hb_lock_init()
 {
     hb_lock_t * l = calloc( sizeof( hb_lock_t ), 1 );
 
-#if defined( SYS_BEOS )
-    l->sem = create_sem( 1, "sem" );
-#elif USE_PTHREAD
     pthread_mutexattr_t mta;
 
     pthread_mutexattr_init(&mta);
 
-#if defined( SYS_CYGWIN ) || defined( SYS_FREEBSD ) || defined ( __FreeBSD__ )
+#if defined( SYS_CYGWIN ) || defined( SYS_FREEBSD ) || defined( SYS_NETBSD ) || \
+    defined( SYS_OPENBSD )
     pthread_mutexattr_settype(&mta, PTHREAD_MUTEX_NORMAL);
 #endif
 
     pthread_mutex_init( &l->mutex, &mta );
-//#elif defined( SYS_CYGWIN )
-//    l->mutex = CreateMutex( 0, FALSE, 0 );
-#endif
 
     return l;
 }
@@ -1014,13 +996,7 @@ void hb_lock_close( hb_lock_t ** _l )
     {
         return;
     }
-#if defined( SYS_BEOS )
-    delete_sem( l->sem );
-#elif USE_PTHREAD
     pthread_mutex_destroy( &l->mutex );
-//#elif defined( SYS_CYGWIN )
-//    CloseHandle( l->mutex );
-#endif
     free( l );
 
     *_l = NULL;
@@ -1028,24 +1004,12 @@ void hb_lock_close( hb_lock_t ** _l )
 
 void hb_lock( hb_lock_t * l )
 {
-#if defined( SYS_BEOS )
-    acquire_sem( l->sem );
-#elif USE_PTHREAD
     pthread_mutex_lock( &l->mutex );
-//#elif defined( SYS_CYGWIN )
-//    WaitForSingleObject( l->mutex, INFINITE );
-#endif
 }
 
 void hb_unlock( hb_lock_t * l )
 {
-#if defined( SYS_BEOS )
-    release_sem( l->sem );
-#elif USE_PTHREAD
     pthread_mutex_unlock( &l->mutex );
-//#elif defined( SYS_CYGWIN )
-//    ReleaseMutex( l->mutex );
-#endif
 }
 
 /************************************************************************
@@ -1053,13 +1017,7 @@ void hb_unlock( hb_lock_t * l )
  ***********************************************************************/
 struct hb_cond_s
 {
-#if defined( SYS_BEOS )
-    int                 thread;
-#elif USE_PTHREAD
     pthread_cond_t      cond;
-//#elif defined( SYS_CYGWIN )
-//    HANDLE              event;
-#endif
 };
 
 /************************************************************************
@@ -1078,14 +1036,7 @@ hb_cond_t * hb_cond_init()
     if( c == NULL )
         return NULL;
 
-#if defined( SYS_BEOS )
-    c->thread = -1;
-#elif USE_PTHREAD
     pthread_cond_init( &c->cond, NULL );
-//#elif defined( SYS_CYGWIN )
-//    c->event = CreateEvent( NULL, FALSE, FALSE, NULL );
-#endif
-
     return c;
 }
 
@@ -1097,12 +1048,7 @@ void hb_cond_close( hb_cond_t ** _c )
     {
         return;
     }
-#if defined( SYS_BEOS )
-#elif USE_PTHREAD
     pthread_cond_destroy( &c->cond );
-//#elif defined( SYS_CYGWIN )
-//    CloseHandle( c->event );
-#endif
     free( c );
 
     *_c = NULL;
@@ -1110,18 +1056,7 @@ void hb_cond_close( hb_cond_t ** _c )
 
 void hb_cond_wait( hb_cond_t * c, hb_lock_t * lock )
 {
-#if defined( SYS_BEOS )
-    c->thread = find_thread( NULL );
-    release_sem( lock->sem );
-    suspend_thread( c->thread );
-    acquire_sem( lock->sem );
-    c->thread = -1;
-#elif USE_PTHREAD
     pthread_cond_wait( &c->cond, &lock->mutex );
-//#elif defined( SYS_CYGWIN )
-//    SignalObjectAndWait( lock->mutex, c->event, INFINITE, FALSE );
-//    WaitForSingleObject( lock->mutex, INFINITE );
-#endif
 }
 
 void hb_clock_gettime( struct timespec *tp )
@@ -1140,51 +1075,22 @@ void hb_yield(void)
 
 void hb_cond_timedwait( hb_cond_t * c, hb_lock_t * lock, int msec )
 {
-#if defined( SYS_BEOS )
-    c->thread = find_thread( NULL );
-    release_sem( lock->sem );
-    suspend_thread( c->thread );
-    acquire_sem( lock->sem );
-    c->thread = -1;
-#elif USE_PTHREAD
     struct timespec ts;
     hb_clock_gettime(&ts);
     ts.tv_nsec += (msec % 1000) * 1000000;
     ts.tv_sec += msec / 1000 + (ts.tv_nsec / 1000000000);
     ts.tv_nsec %= 1000000000;
     pthread_cond_timedwait( &c->cond, &lock->mutex, &ts );
-#endif
 }
 
 void hb_cond_signal( hb_cond_t * c )
 {
-#if defined( SYS_BEOS )
-    while( c->thread != -1 )
-    {
-        thread_info info;
-        get_thread_info( c->thread, &info );
-        if( info.state == B_THREAD_SUSPENDED )
-        {
-            resume_thread( c->thread );
-            break;
-        }
-        /* Looks like we have been called between hb_cond_wait's
-           release_sem() and suspend_thread() lines. Wait until the
-           thread is actually suspended before we resume it */
-        snooze( 5000 );
-    }
-#elif USE_PTHREAD
     pthread_cond_signal( &c->cond );
-//#elif defined( SYS_CYGWIN )
-//    PulseEvent( c->event );
-#endif
 }
 
 void hb_cond_broadcast( hb_cond_t * c )
 {
-#if USE_PTHREAD
     pthread_cond_broadcast( &c->cond );
-#endif
 }
 
 /************************************************************************
@@ -1346,7 +1252,7 @@ void hb_system_sleep_private_disable(void *opaque)
         hb_error("hb_system_sleep: opaque is NULL");
         return;
     }
-    
+
     IOPMAssertionID *assertionID = (IOPMAssertionID*)opaque;
     if (*assertionID != -1)
     {
@@ -1378,7 +1284,7 @@ void hb_system_sleep_private_disable(void *opaque)
 void * hb_dlopen(const char *name)
 {
 #ifdef SYS_MINGW
-    HMODULE h = LoadLibraryA(name);
+    HMODULE h = LoadLibraryExA(name, NULL, LOAD_LIBRARY_SEARCH_SYSTEM32);
 #else
     void *h = dlopen(name, RTLD_LAZY | RTLD_LOCAL);
 #endif
@@ -1405,3 +1311,304 @@ int hb_dlclose(void *h)
 #endif
 }
 
+size_t hb_getline(char ** lineptr, size_t * n, FILE * fp)
+{
+#ifdef SYS_MINGW
+    char   * bufptr = NULL;
+    char   * p      = bufptr;
+    size_t   size;
+    int      c;
+
+    if (lineptr == NULL)
+    {
+        return -1;
+    }
+    if (fp == NULL)
+    {
+        return -1;
+    }
+    if (n == NULL)
+    {
+        return -1;
+    }
+    bufptr = *lineptr;
+    size   = *n;
+
+    c = fgetc(fp);
+    if (c == EOF)
+    {
+        return -1;
+    }
+    if (bufptr == NULL)
+    {
+        bufptr = malloc(128);
+        if (bufptr == NULL)
+        {
+            return -1;
+        }
+        size = 128;
+    }
+    p = bufptr;
+    while (c != EOF)
+    {
+        if ((p - bufptr) >= (size - 1))
+        {
+            char * tmp;
+            size = size + 128;
+            tmp = realloc(bufptr, size);
+            if (tmp == NULL)
+            {
+                free(bufptr);
+                return -1;
+            }
+            p = tmp + (p - bufptr);
+            bufptr = tmp;
+        }
+        *p++ = c;
+        if (c == '\n')
+        {
+            break;
+        }
+        c = fgetc(fp);
+    }
+
+    *p++ = '\0';
+    *lineptr = bufptr;
+    *n = size;
+
+    return p - bufptr - 1;
+#else
+    return getline(lineptr, n, fp);
+#endif
+}
+
+char * hb_strndup(const char * src, size_t len)
+{
+#ifdef SYS_MINGW
+    char * result, * end;
+
+    if (src == NULL)
+    {
+        return NULL;
+    }
+
+    end = memchr(src, 0, len);
+    if (end != NULL)
+    {
+        len = end - src;
+    }
+
+    result = malloc(len + 1);
+    if (result == NULL)
+    {
+        return NULL;
+    }
+    memcpy(result, src, len);
+    result[len] = 0;
+
+    return result;
+#else
+    return strndup(src, len);
+#endif
+}
+
+#if HB_PROJECT_FEATURE_QSV
+#if defined(SYS_LINUX) || defined(SYS_FREEBSD)
+
+#define MAX_NODES             16
+#define DRI_RENDER_NODE_START 128
+#define DRI_RENDER_NODE_LAST  (DRI_RENDER_NODE_START + MAX_NODES - 1)
+#define DRI_CARD_NODE_START   0
+#define DRI_CARD_NODE_LAST    (DRI_CARD_NODE_START + MAX_NODES - 1)
+
+const char* DRI_PATH = "/dev/dri/";
+const char* DRI_NODE_RENDER = "renderD";
+const char* DRI_NODE_CARD = "card";
+
+static int try_adapter(const char * name, const char * dir,
+                const char * prefix, int node_start, int node_last)
+{
+    int             node;
+    int             len        = strlen(name);
+    char          * driverName = malloc(len + 1);
+    drm_version_t   version = {};
+
+    version.name_len = len + 1;
+    version.name     = driverName;
+    for (node = node_start; node <= node_last; node++)
+    {
+        char * adapter = hb_strdup_printf("%s%s%d", dir, prefix, node);
+        int    fd      = open(adapter, O_RDWR);
+
+        free(adapter);
+        if (fd < 0)
+        {
+            continue;
+        }
+
+        if (!ioctl(fd, DRM_IOCTL_VERSION, &version) &&
+            version.name_len == len && !strncmp(driverName, name, len))
+        {
+            free(driverName);
+            return fd;
+        }
+        close(fd);
+    }
+
+    free(driverName);
+    return -1;
+}
+
+static int open_adapter(const char * name, const uint dri_render_node)
+{
+    int fd;
+    // If dri_render_node is unknown enumerate across the predifined range of renders
+    if (dri_render_node == 0)
+    {
+        fd = try_adapter(name, DRI_PATH, DRI_NODE_RENDER,
+                            DRI_RENDER_NODE_START, DRI_RENDER_NODE_LAST);
+    }
+    else
+    {
+        fd = try_adapter(name, DRI_PATH, DRI_NODE_RENDER,
+                            dri_render_node, dri_render_node);
+    }
+    if (fd < 0)
+    {
+        fd = try_adapter(name, DRI_PATH, DRI_NODE_CARD,
+                         DRI_CARD_NODE_START, DRI_CARD_NODE_LAST);
+    }
+    return fd;
+}
+
+static int try_va_interface(hb_display_t * hbDisplay,
+                            const char * interface_name)
+{
+    if (interface_name != NULL)
+    {
+        setenv("LIBVA_DRIVER_NAME", interface_name, 1);
+    }
+
+    hbDisplay->vaDisplay = vaGetDisplayDRM(hbDisplay->vaFd);
+    if (hbDisplay->vaDisplay == NULL)
+    {
+        return -1;
+    }
+
+    int major = 0, minor = 0;
+    VAStatus vaRes = vaInitialize(hbDisplay->vaDisplay, &major, &minor);
+    if (vaRes != VA_STATUS_SUCCESS)
+    {
+        vaTerminate(hbDisplay->vaDisplay);
+        return -1;
+    }
+    hbDisplay->handle = hbDisplay->vaDisplay;
+    hbDisplay->mfxType = MFX_HANDLE_VA_DISPLAY;
+
+    return 0;
+}
+
+hb_display_t * hb_display_init(const char         * driver_name,
+                               const uint32_t       dri_render_node,
+                               const char * const * interface_names)
+{
+    hb_display_t * hbDisplay = calloc(sizeof(hb_display_t), 1);
+    char         * env;
+    int            ii;
+
+    hbDisplay->vaDisplay = NULL;
+    hbDisplay->vaFd      = open_adapter(driver_name, dri_render_node);
+    if (hbDisplay->vaFd < 0)
+    {
+        hb_deep_log( 3, "hb_va_display_init: no display found" );
+        free(hbDisplay);
+        return NULL;
+    }
+
+    if ((env = getenv("LIBVA_DRIVER_NAME")) != NULL)
+    {
+        // Use only environment if it's set
+        hb_log("hb_display_init: using VA driver '%s'", env);
+        if (try_va_interface(hbDisplay, NULL) != 0)
+        {
+            close(hbDisplay->vaFd);
+            free(hbDisplay);
+            return NULL;
+        }
+    }
+    else
+    {
+        // Try list of VA driver names
+        for (ii = 0; interface_names[ii] != NULL; ii++)
+        {
+            hb_log("hb_display_init: attempting VA driver '%s'",
+                   interface_names[ii]);
+            if (try_va_interface(hbDisplay, interface_names[ii]) == 0)
+            {
+                return hbDisplay;
+            }
+        }
+        // Try default
+        unsetenv("LIBVA_DRIVER_NAME");
+        hb_log("hb_display_init: attempting VA default driver");
+        if (try_va_interface(hbDisplay, NULL) != 0)
+        {
+            close(hbDisplay->vaFd);
+            free(hbDisplay);
+            return NULL;
+        }
+    }
+    return hbDisplay;
+}
+
+void hb_display_close(hb_display_t ** _d)
+{
+    hb_display_t * hbDisplay = *_d;
+
+    if (hbDisplay == NULL)
+    {
+        return;
+    }
+    if (hbDisplay->vaDisplay)
+    {
+        vaTerminate(hbDisplay->vaDisplay);
+    }
+    if (hbDisplay->vaFd >= 0)
+    {
+        close(hbDisplay->vaFd);
+    }
+    free(hbDisplay);
+
+    *_d = NULL;
+}
+
+#else // !SYS_LINUX && !SYS_FREEBSD
+
+hb_display_t * hb_display_init(const char         * driver_name,
+                               const uint32_t       dri_render_node,
+                               const char * const * interface_names)
+{
+    return NULL;
+}
+
+void hb_display_close(hb_display_t ** _d)
+{
+    (void)_d;
+}
+
+#endif // SYS_LINUX || SYS_FREEBSD
+#else // !HB_PROJECT_FEATURE_QSV
+
+hb_display_t * hb_display_init(const char         * driver_name,
+                               const uint32_t       dri_render_node,
+                               const char * const * interface_names)
+{
+    return NULL;
+}
+
+void hb_display_close(hb_display_t ** _d)
+{
+    (void)_d;
+}
+
+#endif // HB_PROJECT_FEATURE_QSV
