@@ -7,10 +7,11 @@
 #import "HBController.h"
 #import "HBAppDelegate.h"
 #import "HBFocusRingView.h"
-#import "HBToolbarBadgedItem.h"
+#import "HBControllerToolbarDelegate.h"
 #import "HBQueueController.h"
 #import "HBTitleSelectionController.h"
 #import "NSWindow+HBAdditions.h"
+#import "NSToolbar+HBAdditions.h"
 
 #import "HBQueue.h"
 #import "HBQueueWorker.h"
@@ -46,6 +47,12 @@ static void *HBControllerLogLevelContext = &HBControllerLogLevelContext;
 @property (nonatomic, readonly, strong) HBCore *core;
 @property (nonatomic, readonly, strong) HBAppDelegate *delegate;
 
+@property (nonatomic, strong) NSArray<HBSecurityAccessToken *> *fileTokens;
+@property (nonatomic, strong) NSArray<NSURL *> *subsURLs;
+@property (nonatomic, strong) NSURL *destinationFolderURL;
+@property (nonatomic, strong) HBSecurityAccessToken *destinationFolderToken;
+
+
 @property (nonatomic, weak) IBOutlet NSTextField *sourceLabel;
 @property (nonatomic, weak) IBOutlet NSPopUpButton *titlePopUp;
 @property (nonatomic, weak) IBOutlet NSPathControl *destinationPathControl;
@@ -105,8 +112,6 @@ static void *HBControllerLogLevelContext = &HBControllerLogLevelContext;
 
 #pragma mark - Job
 
-@property (nonatomic, strong) NSURL *destinationFolderURL;
-
 @property (nonatomic, nullable) HBJob *job;
 @property (nonatomic, nullable) HBAutoNamer *autoNamer;
 
@@ -122,12 +127,7 @@ static void *HBControllerLogLevelContext = &HBControllerLogLevelContext;
 
 #pragma mark - Toolbar
 
-@property (nonatomic) IBOutlet NSToolbarItem *openSourceToolbarItem;
-@property (nonatomic) IBOutlet NSToolbarItem *ripToolbarItem;
-@property (nonatomic) IBOutlet NSToolbarItem *pauseToolbarItem;
-@property (nonatomic) IBOutlet NSToolbarItem *presetsItem;
-
-@property (nonatomic, weak) IBOutlet HBToolbarBadgedItem *showQueueToolbarItem;
+@property (nonatomic) HBControllerToolbarDelegate *toolbarDelegate;
 
 @end
 
@@ -178,7 +178,7 @@ static void *HBControllerLogLevelContext = &HBControllerLogLevelContext;
         }
 
 #ifdef __SANDBOX_ENABLED__
-        [_destinationFolderURL startAccessingSecurityScopedResource];
+        _destinationFolderToken = [HBSecurityAccessToken tokenWithObject:_destinationFolderURL];
 #endif
     }
 
@@ -194,6 +194,15 @@ static void *HBControllerLogLevelContext = &HBControllerLogLevelContext;
         self.window.toolbarStyle = NSWindowToolbarStyleExpanded;
     }
 
+    self.toolbarDelegate = [[HBControllerToolbarDelegate alloc] init];
+
+    NSToolbar *toolbar = [[NSToolbar alloc] initWithIdentifier:@"HBMainWindowToolbar2"];
+    toolbar.delegate = self.toolbarDelegate;
+    toolbar.allowsUserCustomization = YES;
+    toolbar.autosavesConfiguration = YES;
+    toolbar.displayMode = NSToolbarDisplayModeIconAndLabel;
+    self.window.toolbar = toolbar;
+
     [self enableUI:NO];
 
     // Bottom
@@ -317,12 +326,21 @@ static void *HBControllerLogLevelContext = &HBControllerLogLevelContext;
 - (BOOL)performDragOperation:(id <NSDraggingInfo>)sender
 {
     NSArray<NSURL *> *fileURLs = [self fileURLsFromPasteboard:[sender draggingPasteboard]];
-
     if (fileURLs.count)
     {
-        [self openURL:fileURLs.firstObject];
+        NSArray<NSURL *> *subtitlesFileURLs = [HBUtilities extractURLs:fileURLs withExtension:HBUtilities.supportedExtensions];
+        if (subtitlesFileURLs.count == fileURLs.count && self.job)
+        {
+            [self.subtitlesViewController addTracksFromExternalFiles:fileURLs];
+            NSUInteger index = [self.mainTabView indexOfTabViewItem:self.subtitlesTab];
+            [self.mainTabView selectTabViewItemAtIndex:index];
+        }
+        else
+        {
+            BOOL recursive = [NSUserDefaults.standardUserDefaults boolForKey:HBRecursiveScan];
+            [self openURLs:fileURLs recursive:recursive];
+        }
     }
-
     [self.window.contentView setShowFocusRing:NO];
     return YES;
 }
@@ -339,7 +357,7 @@ static void *HBControllerLogLevelContext = &HBControllerLogLevelContext;
     if (context == HBControllerScanCoreContext)
     {
         HBState state = [change[NSKeyValueChangeNewKey] intValue];
-        [self updateToolbarButtonsStateForScanCore:state];
+        [self.toolbarDelegate updateToolbarButtonsStateForScanCore:state toolbar:self.window.toolbar];
         [self _touchBar_updateButtonsStateForScanCore:state];
         [self _touchBar_validateUserInterfaceItems];
     }
@@ -355,59 +373,14 @@ static void *HBControllerLogLevelContext = &HBControllerLogLevelContext;
 
 - (void)updateQueueUI
 {
-    [self updateToolbarButtonsState];
+    [self.toolbarDelegate updateToolbarButtonsState:self.queue toolbar:self.window.toolbar];
     [self.window.toolbar validateVisibleItems];
 
     [self _touchBar_updateQueueButtonsState];
     [self _touchBar_validateUserInterfaceItems];
 
     NSUInteger count = self.queue.pendingItemsCount;
-    self.showQueueToolbarItem.badgeValue = count ? @(count).stringValue : @"";
-}
-
-- (void)updateToolbarButtonsStateForScanCore:(HBState)state
-{
-    if (state == HBStateIdle)
-    {
-        _openSourceToolbarItem.image = [NSImage imageNamed: @"source"];
-        _openSourceToolbarItem.label = NSLocalizedString(@"Open Source",  @"Toolbar Open/Cancel Item");
-        _openSourceToolbarItem.toolTip = NSLocalizedString(@"Open Source", @"Toolbar Open/Cancel Item");
-    }
-    else
-    {
-        _openSourceToolbarItem.image = [NSImage imageNamed: @"stopencode"];
-        _openSourceToolbarItem.label = NSLocalizedString(@"Cancel Scan", @"Toolbar Open/Cancel Item");
-        _openSourceToolbarItem.toolTip = NSLocalizedString(@"Cancel Scanning Source", @"Toolbar Open/Cancel Item");
-    }
-}
-
-- (void)updateToolbarButtonsState
-{
-    if (self.queue.canResume)
-    {
-        _pauseToolbarItem.image = [NSImage imageNamed: @"encode"];
-        _pauseToolbarItem.label = NSLocalizedString(@"Resume", @"Toolbar Pause Item");
-        _pauseToolbarItem.toolTip = NSLocalizedString(@"Resume Encoding", @"Toolbar Pause Item");
-    }
-    else
-    {
-        _pauseToolbarItem.image = [NSImage imageNamed:@"pauseencode"];
-        _pauseToolbarItem.label = NSLocalizedString(@"Pause", @"Toolbar Pause Item");
-        _pauseToolbarItem.toolTip = NSLocalizedString(@"Pause Encoding", @"Toolbar Pause Item");
-
-    }
-    if (self.queue.isEncoding)
-    {
-        _ripToolbarItem.image = [NSImage imageNamed:@"stopencode"];
-        _ripToolbarItem.label = NSLocalizedString(@"Stop", @"Toolbar Start/Stop Item");
-        _ripToolbarItem.toolTip = NSLocalizedString(@"Stop Encoding", @"Toolbar Start/Stop Item");
-    }
-    else
-    {
-        _ripToolbarItem.image = [NSImage imageNamed: @"encode"];
-        _ripToolbarItem.label = _queue.pendingItemsCount > 0 ? NSLocalizedString(@"Start Queue", @"Toolbar Start/Stop Item") :  NSLocalizedString(@"Start", @"Toolbar Start/Stop Item");
-        _ripToolbarItem.toolTip = NSLocalizedString(@"Start Encoding", @"Toolbar Start/Stop Item");
-    }
+    [self.toolbarDelegate updateToolbarQueueBadge:count ? @(count).stringValue : @"" toolbar:self.window.toolbar];
 }
 
 - (void)enableUI:(BOOL)enabled
@@ -548,7 +521,8 @@ static void *HBControllerLogLevelContext = &HBControllerLogLevelContext;
         }
     }
 
-    if (action == @selector(togglePauseResume:)) {
+    if (action == @selector(togglePauseResume:))
+    {
         return self.queue.canPause || self.queue.canResume;
     }
 
@@ -674,25 +648,18 @@ static void *HBControllerLogLevelContext = &HBControllerLogLevelContext;
 /**
  * Here we actually tell hb_scan to perform the source scan, using the path to source and title number
  */
-- (void)scanURL:(NSURL *)fileURL titleIndex:(NSUInteger)index completionHandler:(void(^)(NSArray<HBTitle *> *titles))completionHandler
+- (void)scanURLs:(NSArray<NSURL *> *)fileURLs titleIndex:(NSUInteger)index completionHandler:(void(^)(NSArray<HBTitle *> *titles))completionHandler
 {
-    // Save the current settings
-    [self updateCurrentPreset];
-
-    self.job = nil;
-    [self.titlePopUp removeAllItems];
-    self.window.representedURL = nil;
-    self.window.title = NSLocalizedString(@"HandBrake", @"Main Window -> title");
-
-    // Clear the undo manager, we can't undo this action
-    [self.window.undoManager removeAllActions];
-
-    NSURL *mediaURL = [HBUtilities mediaURLFromURL:fileURL];
-
-    NSError *outError = NULL;
+    [self showWindow:self];
 
     // Check if we can scan the source and if there is any warning.
-    BOOL canScan = [self.core canScan:mediaURL error:&outError];
+    NSError *outError = NULL;
+    BOOL canScan = YES;
+
+    if (fileURLs.count == 1)
+    {
+        canScan = [self.core canScan:fileURLs error:&outError];
+    }
 
     // Notify the user that we don't support removal of copy protection.
     if (canScan && outError.code == 101 && !self.suppressCopyProtectionWarning)
@@ -716,10 +683,12 @@ static void *HBControllerLogLevelContext = &HBControllerLogLevelContext;
         NSUInteger hb_num_previews = [NSUserDefaults.standardUserDefaults integerForKey:HBPreviewsNumber];
         NSUInteger min_title_duration_seconds = [NSUserDefaults.standardUserDefaults integerForKey:HBMinTitleScanSeconds];
 
-        [self.core scanURL:mediaURL
-                titleIndex:index
-                  previews:hb_num_previews minDuration:min_title_duration_seconds keepPreviews:YES
-           progressHandler:^(HBState state, HBProgress progress, NSString *info)
+        [self.core scanURLs:fileURLs
+                 titleIndex:index
+                   previews:hb_num_previews minDuration:min_title_duration_seconds
+               keepPreviews:YES
+            hardwareDecoder:[NSUserDefaults.standardUserDefaults boolForKey:HBUseHardwareDecoder]
+            progressHandler:^(HBState state, HBProgress progress, NSString *info)
          {
              self.sourceLabel.stringValue = info;
              self.scanIndicator.hidden = NO;
@@ -739,8 +708,6 @@ static void *HBControllerLogLevelContext = &HBControllerLogLevelContext;
                  {
                      [self.titlePopUp addItemWithTitle:title.description];
                  }
-                 self.window.representedURL = mediaURL;
-                 self.window.title = mediaURL.lastPathComponent;
              }
              else
              {
@@ -748,21 +715,11 @@ static void *HBControllerLogLevelContext = &HBControllerLogLevelContext;
                  self.sourceLabel.stringValue = NSLocalizedString(@"No Valid Source Found", @"Main Window -> Info text");
              }
 
-             // Set the last searched source directory in the prefs here
-             if ([NSWorkspace.sharedWorkspace isFilePackageAtPath:mediaURL.URLByDeletingLastPathComponent.path])
-             {
-                 [NSUserDefaults.standardUserDefaults setURL:mediaURL.URLByDeletingLastPathComponent.URLByDeletingLastPathComponent forKey:HBLastSourceDirectoryURL];
-             }
-             else
-             {
-                 [NSUserDefaults.standardUserDefaults setURL:mediaURL.URLByDeletingLastPathComponent forKey:HBLastSourceDirectoryURL];
-             }
-
              completionHandler(self.core.titles);
 
-             // Clear the undo manager, the completion handle
+             // Clear the undo manager, the completion handler
              // set the job in the main window
-             // and don't want to make it undoable
+             // and we don't want to make it undoable
              [self.window.undoManager removeAllActions];
              [self.window.toolbar validateVisibleItems];
              [self _touchBar_validateUserInterfaceItems];
@@ -774,15 +731,94 @@ static void *HBControllerLogLevelContext = &HBControllerLogLevelContext;
     }
 }
 
-- (void)openURL:(NSURL *)fileURL titleIndex:(NSUInteger)index
+- (void)showOpenPanelForDestination:(NSURL *)destinationURL
 {
-    [self showWindow:self];
+    NSOpenPanel *panel = [NSOpenPanel openPanel];
+    panel.canChooseFiles = NO;
+    panel.canChooseDirectories = YES;
+    panel.directoryURL = destinationURL;
+    panel.message = NSLocalizedString(@"HandBrake does not have permission to write to this folder. To allow HandBrake to write to this folder, click \"Allow\"", @"Main Window -> Same as source destination open panel");
+    panel.prompt = NSLocalizedString(@"Allow", @"Main Window -> Same as source destination open panel");
 
-    [self scanURL:fileURL titleIndex:index completionHandler:^(NSArray<HBTitle *> *titles)
+    [panel beginSheetModalForWindow:self.window completionHandler:^(NSInteger result)
+     {
+         if (result == NSModalResponseOK)
+         {
+             self.destinationFolderURL = panel.URL;
+             self.destinationFolderToken = [HBSecurityAccessToken tokenWithAlreadyAccessedObject:panel.URL];
+         }
+     }];
+}
+
+- (void)askForPermissionAndSetDestinationURLs:(NSArray<NSURL *> *)destinationURLs
+{
+    if (destinationURLs.count == 0)
     {
+        return;
+    }
+
+    if (![self.destinationFolderURL isEqualTo:destinationURLs.firstObject])
+    {
+#ifdef __SANDBOX_ENABLED__
+            [self showOpenPanelForDestination:destinationURLs.firstObject];
+#else
+            self.destinationFolderURL = destinationURLs.firstObject;
+#endif
+    }
+}
+
+- (void)cleanUp
+{
+    self.job = nil;
+    self.fileTokens = nil;
+    self.subsURLs = nil;
+
+    [self.titlePopUp removeAllItems];
+    self.window.representedURL = nil;
+    self.window.title = NSLocalizedString(@"HandBrake", @"Main Window -> title");
+
+    // Clear the undo manager, we can't undo this action
+    [self.window.undoManager removeAllActions];
+}
+
+- (void)openURLs:(NSArray<NSURL *> *)fileURLs recursive:(BOOL)recursive titleIndex:(NSUInteger)index
+{
+    // Save the current settings
+    [self updateCurrentPreset];
+    [self cleanUp];
+
+    NSMutableArray<HBSecurityAccessToken *> *tokens = [[NSMutableArray alloc] init];
+    for (NSURL *fileURL in fileURLs)
+    {
+        [tokens addObject:[HBSecurityAccessToken tokenWithAlreadyAccessedObject:fileURL]];
+    }
+    self.fileTokens = tokens;
+
+    NSMutableArray<NSString *> *excludedExtensions = [[NSMutableArray alloc] init];
+    for (NSString *extension in [NSUserDefaults.standardUserDefaults arrayForKey:HBExcludedFileExtensions])
+    {
+        // Make sure there are only NSString instances in the array
+        // Third parties can write to user defaults too and add different kind of objects.
+        if ([extension isKindOfClass:[NSString class]])
+        {
+            [excludedExtensions addObject:extension];
+        }
+    }
+
+    NSArray<NSURL *> *expandedFileURLs  = [HBUtilities expandURLs:fileURLs recursive:recursive];
+    NSArray<NSURL *> *subtitlesFileURLs = [HBUtilities extractURLs:expandedFileURLs withExtension:HBUtilities.supportedExtensions];
+    NSArray<NSURL *> *trimmedFileURLs   = [HBUtilities trimURLs:expandedFileURLs withExtension:excludedExtensions];
+
+    [self scanURLs:trimmedFileURLs titleIndex:index completionHandler:^(NSArray<HBTitle *> *titles)
+    {
+        NSArray<NSURL *> *baseURLs = [HBUtilities baseURLs:trimmedFileURLs];
+
         if (titles.count)
         {
-            [NSDocumentController.sharedDocumentController noteNewRecentDocumentURL:fileURL];
+            for (NSURL *fileURL in fileURLs)
+            {
+                [NSDocumentController.sharedDocumentController noteNewRecentDocumentURL:fileURL];
+            }
 
             HBTitle *featuredTitle = titles.firstObject;
             for (HBTitle *title in titles)
@@ -793,26 +829,34 @@ static void *HBControllerLogLevelContext = &HBControllerLogLevelContext;
                 }
             }
 
+            self.subsURLs = subtitlesFileURLs;
+
             HBJob *job = [self jobFromTitle:featuredTitle];
             if (job)
             {
                 self.job = job;
+                if (featuredTitle.isStream && [NSUserDefaults.standardUserDefaults boolForKey:HBUseSourceFolderDestination])
+                {
+                    [self askForPermissionAndSetDestinationURLs:baseURLs];
+                }
             }
             else
             {
-                self.job = nil;
-                [self.titlePopUp removeAllItems];
+                [self cleanUp];
                 self.sourceLabel.stringValue = NSLocalizedString(@"No Valid Preset", @"Main Window -> Info text");
             }
         }
+
+        // Set the last searched source directory in the prefs here
+        [NSUserDefaults.standardUserDefaults setURL:baseURLs.firstObject forKey:HBLastSourceDirectoryURL];
     }];
 }
 
-- (void)openURL:(NSURL *)fileURL
+- (void)openURLs:(NSArray<NSURL *> *)fileURLs recursive:(BOOL)recursive
 {
     if (self.core.state != HBStateScanning)
     {
-        [self openURL:fileURL titleIndex:0];
+        [self openURLs:fileURLs recursive:recursive titleIndex:0];
     }
 }
 
@@ -823,28 +867,15 @@ static void *HBControllerLogLevelContext = &HBControllerLogLevelContext;
 {
     if (self.core.state != HBStateScanning)
     {
+        [self cleanUp];
         [job refreshSecurityScopedResources];
-        [self scanURL:job.fileURL titleIndex:job.titleIdx completionHandler:^(NSArray<HBTitle *> *titles)
+        self.fileTokens = @[[HBSecurityAccessToken tokenWithObject:job.fileURL]];
+
+        [self scanURLs:@[job.fileURL] titleIndex:job.titleIdx completionHandler:^(NSArray<HBTitle *> *titles)
         {
             if (titles.count)
             {
-                // If the scan was cached, reselect
-                // the original title
-                for (HBTitle *title in titles)
-                {
-                    if (title.index == job.titleIdx)
-                    {
-                        job.title = title;
-                        break;
-                    }
-                }
-
-                // Else just one title or a title specific rescan
-                // select the first title
-                if (!job.title)
-                {
-                    job.title = titles.firstObject;
-                }
+                job.title = titles.firstObject;
 
                 self.job = job;
                 job.undo = self.window.undoManager;
@@ -857,9 +888,9 @@ static void *HBControllerLogLevelContext = &HBControllerLogLevelContext;
             else
             {
                 handler(NO);
+                [self cleanUp];
             }
         }];
-        [self showWindow:self];
     }
     else
     {
@@ -867,16 +898,34 @@ static void *HBControllerLogLevelContext = &HBControllerLogLevelContext;
     }
 }
 
+- (NSArray<NSURL *> *)matchSubtitlesURLsWith:(NSURL *)sourceURL
+{
+    NSMutableArray<NSURL *> *URLs = [[NSMutableArray alloc] init];
+    NSString *sourcePrefix = [sourceURL.path stringByDeletingPathExtension];
+
+    for (NSURL *url in self.subsURLs)
+    {
+        if ([url.path hasPrefix:sourcePrefix])
+        {
+            [URLs addObject:url];
+        }
+    }
+
+    return URLs;
+}
+
 - (HBJob *)jobFromTitle:(HBTitle *)title
 {
-    // If there is already a title load, save the current settings to a preset
-    // Save the current settings
+    // If there is already a title loaded, save the current settings to a preset
     [self updateCurrentPreset];
 
-    HBJob *job = [[HBJob alloc] initWithTitle:title preset:self.currentPreset];
+    NSArray<NSURL *> *subtitlesURLs = [self matchSubtitlesURLsWith:title.url];
+    HBJob *job = [[HBJob alloc] initWithTitle:title preset:self.currentPreset subtitles:subtitlesURLs];
+
     if (job)
     {
-        job.destinationFolderURL = self.destinationFolderURL;
+        [job setDestinationFolderURL:self.destinationFolderURL
+                        sameAsSource:[NSUserDefaults.standardUserDefaults boolForKey:HBUseSourceFolderDestination]];
 
         // If the source is not a stream, and autonaming is disabled,
         // keep the existing file name.
@@ -967,6 +1016,9 @@ static void *HBControllerLogLevelContext = &HBControllerLogLevelContext;
         {
             self.sourceLabel.stringValue = [NSString stringWithFormat:@"%@, %@", title.name, title.shortFormatDescription];
         }
+
+        self.window.representedURL = job.fileURL;
+        self.window.title = job.fileURL.lastPathComponent;
     }
     else
     {
@@ -992,7 +1044,7 @@ static void *HBControllerLogLevelContext = &HBControllerLogLevelContext;
     }
 
     NSOpenPanel *panel = [NSOpenPanel openPanel];
-    [panel setAllowsMultipleSelection:NO];
+    [panel setAllowsMultipleSelection:YES];
     [panel setCanChooseFiles:YES];
     [panel setCanChooseDirectories:YES];
 
@@ -1015,8 +1067,9 @@ static void *HBControllerLogLevelContext = &HBControllerLogLevelContext;
     {
         if (result == NSModalResponseOK)
          {
+             BOOL recursive = [NSUserDefaults.standardUserDefaults boolForKey:HBRecursiveScan];
              NSInteger titleIdx = self.scanSpecificTitle ? self.scanSpecificTitleIdx : 0;
-             [self openURL:panel.URL titleIndex:titleIdx];
+             [self openURLs:panel.URLs recursive:recursive titleIndex:titleIdx];
          }
      }];
 }
@@ -1033,8 +1086,6 @@ static void *HBControllerLogLevelContext = &HBControllerLogLevelContext;
                                               forKey:HBLastDestinationDirectoryBookmark];
     [NSUserDefaults.standardUserDefaults setURL:destinationFolderURL
                                          forKey:HBLastDestinationDirectoryURL];
-
-
 }
 
 - (IBAction)browseDestination:(id)sender
@@ -1056,6 +1107,7 @@ static void *HBControllerLogLevelContext = &HBControllerLogLevelContext;
          if (result == NSModalResponseOK)
          {
              self.destinationFolderURL = panel.URL;
+             self.destinationFolderToken = [HBSecurityAccessToken tokenWithAlreadyAccessedObject:panel.URL];
          }
      }];
 }
@@ -1086,6 +1138,7 @@ static void *HBControllerLogLevelContext = &HBControllerLogLevelContext;
         if (URL.hasDirectoryPath)
         {
             self.destinationFolderURL = URL;
+            self.destinationFolderToken = [HBSecurityAccessToken tokenWithAlreadyAccessedObject:URL];
         }
         return YES;
     }
@@ -1319,6 +1372,7 @@ static void *HBControllerLogLevelContext = &HBControllerLogLevelContext;
     NSMutableArray<HBJob *> *jobs = [[NSMutableArray alloc] init];
     BOOL fileExists = NO;
     BOOL fileOverwritesSource = NO;
+    BOOL useSourceFolderDestination = [NSUserDefaults.standardUserDefaults boolForKey:HBUseSourceFolderDestination];
 
     // Get the preset from the loaded job.
     HBPreset *preset = [self createPresetFromCurrentSettings];
@@ -1326,7 +1380,10 @@ static void *HBControllerLogLevelContext = &HBControllerLogLevelContext;
     for (HBTitle *title in titles)
     {
         HBJob *job = [[HBJob alloc] initWithTitle:title preset:preset];
-        job.destinationFolderURL = self.destinationFolderURL;
+
+        [job setDestinationFolderURL:self.destinationFolderURL
+                        sameAsSource:useSourceFolderDestination];
+
         job.destinationFileName = job.defaultName;
         job.title = nil;
         if (job)
@@ -1439,25 +1496,50 @@ static void *HBControllerLogLevelContext = &HBControllerLogLevelContext;
 
 - (BOOL)popoverShouldDetach:(NSPopover *)popover
 {
-    if (popover == self.presetsPopover) {
+    if (popover == self.presetsPopover)
+    {
         return YES;
     }
 
     return NO;
 }
 
+- (void)popoverDidDetach:(NSPopover *)popover
+{
+    if (popover == self.presetsPopover)
+    {
+        self.presetView.showHeader = YES;
+    }
+}
+
 - (IBAction)togglePresets:(id)sender
 {
-    if (self.presetsPopover)
+    NSToolbarItem *presetsToolbarItem = [self.window.toolbar HB_visibleToolbarItemWithIdentifier:TOOLBAR_PRESET];
+
+    if (self.presetsPopover.isShown)
     {
-        if (!self.presetsPopover.isShown)
+        [self.presetsPopover close];
+    }
+    else
+    {
+        NSView *target = presetsToolbarItem.view.window ? presetsToolbarItem.view : self.window.contentView;
+        if (self.window.toolbar.visible && presetsToolbarItem)
         {
-            NSView *target = [sender isKindOfClass:[NSView class]] ? (NSView *)sender : self.presetsItem.view.window ? self.presetsItem.view : self.window.contentView;
-            [self.presetsPopover showRelativeToRect:target.bounds ofView:target preferredEdge:NSMaxYEdge];
+#ifdef MAC_OS_VERSION_14_0
+            if (@available (macOS 14, *))
+            {
+                [self.presetsPopover showRelativeToToolbarItem:presetsToolbarItem];
+                self.presetView.showHeader = NO;
+            }
+            else
+#endif
+            {
+                [self.presetsPopover showRelativeToRect:target.bounds ofView:target preferredEdge:NSMaxYEdge];
+            }
         }
         else
         {
-            [self.presetsPopover close];
+            [self.presetsPopover showRelativeToRect:target.bounds ofView:target preferredEdge:NSMaxYEdge];
         }
     }
 }
@@ -1702,7 +1784,16 @@ static NSTouchBarItemIdentifier HBTouchBarActivity = @"fr.handbrake.activity";
         NSCustomTouchBarItem *item = [[NSCustomTouchBarItem alloc] initWithIdentifier:identifier];
         item.customizationLabel = NSLocalizedString(@"Add To Queue", @"Touch bar");
 
-        NSButton *button = [NSButton buttonWithTitle:NSLocalizedString(@"Add To Queue", @"Touch bar") target:self action:@selector(addToQueue:)];
+        NSButton *button = nil;
+        if (@available (macOS 11, *))
+        {
+            NSImage *image = [NSImage imageNamed:@"photo.badge.plus"];
+            button = [NSButton buttonWithImage:image target:self action:@selector(addToQueue:)];
+        }
+        else
+        {
+            button = [NSButton buttonWithTitle:NSLocalizedString(@"Add To Queue", @"Touch bar") target:self action:@selector(addToQueue:)];
+        }
 
         item.view = button;
         return item;
@@ -1752,7 +1843,17 @@ static NSTouchBarItemIdentifier HBTouchBarActivity = @"fr.handbrake.activity";
         NSCustomTouchBarItem *item = [[NSCustomTouchBarItem alloc] initWithIdentifier:identifier];
         item.customizationLabel = NSLocalizedString(@"Show Activity Window", @"Touch bar");
 
-        NSButton *button = [NSButton buttonWithImage:[NSImage imageNamed:NSImageNameTouchBarGetInfoTemplate] target:nil action:@selector(showOutputPanel:)];
+        NSImage *image = nil;
+        if (@available (macOS 11, *))
+        {
+            image = [NSImage imageNamed:@"text.viewfinder"];
+        }
+        else
+        {
+            image = [NSImage imageNamed:NSImageNameTouchBarPlayTemplate];
+        }
+
+        NSButton *button = [NSButton buttonWithImage:image target:nil action:@selector(showOutputPanel:)];
 
         item.view = button;
         return item;
