@@ -50,6 +50,15 @@
 
     hb_job_set_file(job, self.destinationURL.fileSystemRepresentation);
 
+    if (self.hwDecodeUsage == HBJobHardwareDecoderUsageFullPathOnly)
+    {
+        job->hw_decode = HB_DECODE_SUPPORT_VIDEOTOOLBOX;
+    }
+    else if (self.hwDecodeUsage == HBJobHardwareDecoderUsageAlways)
+    {
+        job->hw_decode = HB_DECODE_SUPPORT_VIDEOTOOLBOX | HB_DECODE_SUPPORT_FORCE_HW;
+    }
+
     // Title Angle for dvdnav
     job->angle = self.angle;
 
@@ -92,7 +101,7 @@
     job->mux = self.container;
     job->vcodec = self.video.encoder;
 
-    job->mp4_optimize = self.mp4HttpOptimize;
+    job->optimize = self.optimize;
 
     if (self.container & HB_MUX_MASK_MP4)
     {
@@ -138,12 +147,12 @@
         job->ipod_atom = self.mp4iPodCompatible;
     }
 
-    if (self.video.twoPass && ((self.video.encoder & HB_VCODEC_X264_MASK) ||
+    if (self.video.multiPass && ((self.video.encoder & HB_VCODEC_X264_MASK) ||
                                (self.video.encoder & HB_VCODEC_X265_MASK)))
     {
-        job->fastfirstpass = self.video.turboTwoPass;
+        job->fastanalysispass = self.video.turboMultiPass;
     }
-    job->twopass = self.video.twoPass;
+    job->multipass = self.video.multiPass;
 
     if (hb_video_encoder_get_presets(self.video.encoder) != NULL)
     {
@@ -386,7 +395,7 @@
                                    inputTrack.sampleRate :
                                    audioTrack.sampleRate);
 
-            audio->in.track = (int)audioTrack.sourceTrackIdx - 1;
+            audio->index = (int)audioTrack.sourceTrackIdx - 1;
 
             // We go ahead and assign values to our audio->out.<properties>
             audio->out.track                     = audio->in.track;
