@@ -24,12 +24,19 @@ NS_ASSUME_NONNULL_BEGIN
 extern NSString *HBContainerChangedNotification;
 extern NSString *HBChaptersChangedNotification;
 
+typedef NS_ENUM(NSUInteger, HBJobHardwareDecoderUsage) {
+    HBJobHardwareDecoderUsageNone,
+    HBJobHardwareDecoderUsageAlways,
+    HBJobHardwareDecoderUsageFullPathOnly
+};
+
 /**
  * HBJob
  */
 @interface HBJob : NSObject <NSSecureCoding, NSCopying, HBPresetCoding, HBSecurityScope>
 
 - (nullable instancetype)initWithTitle:(HBTitle *)title preset:(HBPreset *)preset;
+- (nullable instancetype)initWithTitle:(HBTitle *)title preset:(HBPreset *)preset subtitles:(NSArray<NSURL *> *)subtitlesURLs;
 
 @property (nonatomic, readwrite, weak, nullable) HBTitle *title;
 @property (nonatomic, readonly) int titleIdx;
@@ -56,7 +63,7 @@ extern NSString *HBChaptersChangedNotification;
 @property (nonatomic, readwrite) int angle;
 
 // Container options
-@property (nonatomic, readwrite) BOOL mp4HttpOptimize;
+@property (nonatomic, readwrite) BOOL optimize;
 @property (nonatomic, readwrite) BOOL mp4iPodCompatible;
 @property (nonatomic, readwrite) BOOL alignAVStart;
 
@@ -72,6 +79,7 @@ extern NSString *HBChaptersChangedNotification;
 @property (nonatomic, readonly) NSArray<HBChapter *> *chapterTitles;
 
 @property (nonatomic, readwrite) BOOL metadataPassthru;
+@property (nonatomic, readwrite) HBJobHardwareDecoderUsage hwDecodeUsage;
 
 @property (nonatomic, readwrite, weak, nullable) NSUndoManager *undo;
 
