@@ -55,9 +55,12 @@ static void *HBPresetsViewControllerContext = &HBPresetsViewControllerContext;
 
 @property (nonatomic, strong) HBPresetsManager *presets;
 @property (nonatomic, readwrite) HBPreset *selectedPresetInternal;
-@property (nonatomic, unsafe_unretained) IBOutlet NSTreeController *treeController;
+@property (nonatomic, weak) IBOutlet NSTreeController *treeController;
 @property (nonatomic, weak) IBOutlet NSSegmentedControl *actionsControl;
 
+@property (nonatomic, strong) IBOutlet NSTextField *headerLabel;
+@property (nonatomic, strong) IBOutlet NSLayoutConstraint *headerBottomConstraint;
+
 /**
  *  Helper var for drag & drop
  */
@@ -84,6 +87,7 @@ static void *HBPresetsViewControllerContext = &HBPresetsViewControllerContext;
         _selectedPresetInternal = presetManager.defaultPreset;
         _expandedNodes = [[NSArray arrayWithArray:[NSUserDefaults.standardUserDefaults
                                                    objectForKey:@"HBPreviewViewExpandedStatus"]] mutableCopy];
+        _showHeader = YES;
     }
     return self;
 }
@@ -100,6 +104,9 @@ static void *HBPresetsViewControllerContext = &HBPresetsViewControllerContext;
     // Re-expand the items
     [self expandNodes:self.treeController.arrangedObjects.childNodes];
 
+    // Update header state
+    self.showHeader = _showHeader;
+
     [self.treeController setSelectionIndexPath:[self.presets indexPathOfPreset:self.selectedPreset]];
     [self.treeController addObserver:self forKeyPath:@"selectedObjects" options:NSKeyValueObservingOptionNew context:HBPresetsViewControllerContext];
 
@@ -196,6 +203,7 @@ static void *HBPresetsViewControllerContext = &HBPresetsViewControllerContext;
                     [self presentError:error];
                 }
             }
+            [panel.URL stopAccessingSecurityScopedResource];
         }
     }];
 }
@@ -282,7 +290,7 @@ static void *HBPresetsViewControllerContext = &HBPresetsViewControllerContext;
     panel.allowsMultipleSelection = YES;
     panel.canChooseFiles = YES;
     panel.canChooseDirectories = NO;
-    panel.allowedFileTypes = @[@"plist", @"xml", @"json"];
+    panel.allowedFileTypes = @[@"json"];
 
     if ([NSUserDefaults.standardUserDefaults URLForKey:@"LastPresetImportDirectoryURL"])
     {
@@ -301,11 +309,24 @@ static void *HBPresetsViewControllerContext = &HBPresetsViewControllerContext;
          {
              [self doImportPreset:panel.URLs atIndexPath:nil];
          }
+
+         for (NSURL *url in panel.URLs)
+         {
+             [url stopAccessingSecurityScopedResource];
+         }
      }];
 }
 
 #pragma mark - UI Methods
 
+- (void)setShowHeader:(BOOL)showHeader
+{
+    _showHeader = showHeader;
+
+    self.headerLabel.hidden = !showHeader;
+    self.headerBottomConstraint.active = showHeader;
+}
+
 - (IBAction)clicked:(id)sender
 {
     if (self.delegate && [self.treeController.selectedObjects.firstObject isLeaf] && [self.treeController.selectedObjects.firstObject isSupported])
@@ -560,7 +581,13 @@ static void *HBPresetsViewControllerContext = &HBPresetsViewControllerContext;
 - (void)handleExternalDrops:(NSPasteboard *)pboard withIndexPath:(NSIndexPath *)indexPath
 {
     NSArray<NSURL *> *URLs = [pboard readObjectsForClasses:@[[NSURL class]] options:nil];
+
     [self doImportPreset:URLs atIndexPath:indexPath];
+
+    for (NSURL *url in URLs)
+    {
+        [url stopAccessingSecurityScopedResource];
+    }
 }
 
 - (BOOL)outlineView:(NSOutlineView *)ov acceptDrop:(id <NSDraggingInfo>)info item:(id)targetItem childIndex:(NSInteger)index
