@@ -85,6 +85,7 @@
 
     [_connection resume];
 
+    [_proxy initGlobal];
     [_proxy setDVDNav:[NSUserDefaults.standardUserDefaults boolForKey:HBUseDvdNav]];
     [_proxy setUpWithLogLevel:self.level name:self.name];
     [_proxy setAutomaticallyPreventSleep:self.automaticallyPreventSleep];
@@ -92,7 +93,7 @@
 
 - (void)invalidate
 {
-    [[_connection synchronousRemoteObjectProxyWithErrorHandler:^(NSError * _Nonnull error) {}] tearDown];
+    [[_connection synchronousRemoteObjectProxyWithErrorHandler:^(NSError * _Nonnull error) {}] closeGlobal];
     [_connection invalidate];
     _connection = nil;
     _proxy = nil;
@@ -100,28 +101,37 @@
 
 - (void)handleInterruption
 {
-    [_proxy setDVDNav:[NSUserDefaults.standardUserDefaults boolForKey:HBUseDvdNav]];
-    [_proxy setUpWithLogLevel:self.level name:self.name];
-    [_proxy setAutomaticallyPreventSleep:self.automaticallyPreventSleep];
+    [_connection invalidate];
+    _connection = nil;
+    _proxy = nil;
 
-    HBCoreCompletionHandler handler = self.completionHandler;
-
-    self.progressHandler = nil;
-    self.completionHandler = nil;
-
-    self.state = HBStateIdle;
-
-    if (handler)
+    if (self.state == HBStateIdle)
     {
-        HBCoreResult result = {0, HBCoreResultCodeUnknown};
-        result.code = HBCoreResultCodeUnknown;
-        handler(result);
+        // XPC was idle and the system closed it
+        // to free ram, no need to restart it.
+        return;
     }
+    else
+    {
+        HBCoreCompletionHandler handler = self.completionHandler;
 
-    [self forwardError:@"XPC: Service did crash\n"];
+        self.progressHandler = nil;
+        self.completionHandler = nil;
+        self.state = HBStateIdle;
+
+        if (handler)
+        {
+            HBCoreResult result = {0, HBCoreResultCodeUnknown};
+            result.code = HBCoreResultCodeUnknown;
+            handler(result);
+        }
+
+        [self forwardError:@"XPC: Service did crash\n"];
+    }
 }
 
-- (void)updateState:(HBState)state {
+- (void)updateState:(HBState)state
+{
     dispatch_sync(dispatch_get_main_queue(), ^{
         self.state = state;
     });
@@ -149,7 +159,7 @@
     [_proxy preventSleep];
 }
 
-- (void)scanURL:(NSURL *)url titleIndex:(NSUInteger)index previews:(NSUInteger)previewsNum minDuration:(NSUInteger)seconds keepPreviews:(BOOL)keepPreviews progressHandler:(nonnull HBCoreProgressHandler)progressHandler completionHandler:(nonnull HBCoreCompletionHandler)completionHandler
+- (void)scanURL:(NSURL *)url titleIndex:(NSUInteger)index previews:(NSUInteger)previewsNum minDuration:(NSUInteger)seconds keepPreviews:(BOOL)keepPreviews hardwareDecoder:(BOOL)hardwareDecoder progressHandler:(nonnull HBCoreProgressHandler)progressHandler completionHandler:(nonnull HBCoreCompletionHandler)completionHandler
 {
     if (!_connection)
     {
@@ -173,7 +183,7 @@
 
     __weak HBRemoteCore *weakSelf = self;
 
-    [_proxy scanURL:url titleIndex:index previews:previewsNum minDuration:seconds keepPreviews:keepPreviews withReply:^(HBCoreResult result) {
+    [_proxy scanURL:url titleIndex:index previews:previewsNum minDuration:seconds keepPreviews:keepPreviews hardwareDecoder:(BOOL)hardwareDecoder withReply:^(HBCoreResult result) {
         dispatch_sync(dispatch_get_main_queue(), ^{
             HBCoreCompletionHandler handler = weakSelf.completionHandler;
             weakSelf.completionHandler = nil;
@@ -210,10 +220,16 @@
         }
     }
 
-    NSData *bookmark = [job.outputURL bookmarkDataWithOptions:0 includingResourceValuesForKeys:nil relativeToURL:nil error:NULL];
-    if (bookmark)
+    NSData *destinationBookmark = [job.destinationFolderURL bookmarkDataWithOptions:0 includingResourceValuesForKeys:nil relativeToURL:nil error:NULL];
+    if (destinationBookmark)
     {
-        [bookmarks addObject:bookmark];
+        [bookmarks addObject:destinationBookmark];
+    }
+
+    NSData *sourceBookmark = [job.fileURL bookmarkDataWithOptions:0 includingResourceValuesForKeys:nil relativeToURL:nil error:NULL];
+    if (sourceBookmark)
+    {
+        [bookmarks addObject:sourceBookmark];
     }
 
     [_proxy provideResourceAccessWithBookmarks:bookmarks];
@@ -245,13 +261,16 @@
     [_proxy cancelEncode];
 }
 
-- (void)updateProgress:(double)currentProgress hours:(int)hours minutes:(int)minutes seconds:(int)seconds state:(HBState)state info:(NSString *)info {
-
+- (void)updateProgress:(double)currentProgress hours:(int)hours minutes:(int)minutes seconds:(int)seconds state:(HBState)state info:(NSString *)info
+{
     __weak HBRemoteCore *weakSelf = self;
 
     dispatch_sync(dispatch_get_main_queue(), ^{
-        HBProgress progress = {currentProgress , hours, minutes, seconds};
-        weakSelf.progressHandler(state, progress, info);
+        if (weakSelf.progressHandler)
+        {
+            HBProgress progress = {currentProgress , hours, minutes, seconds};
+            weakSelf.progressHandler(state, progress, info);
+        }
     });
 }
 
