@@ -79,6 +79,13 @@ NSString * const HBVideoChangedNotification = @"HBVideoChangedNotification";
     // Ensure the quality value is not out of the new bounds
     if (!(self.undo.isUndoing || self.undo.isRedoing))
     {
+        double integral;
+        double fractional = modf(self.quality, &integral);
+
+        if ((int)(fractional * 100) % (int)(granularity * 100))
+        {
+            self.quality = ceil(self.quality);
+        }
         if (self.quality > maxValue)
         {
             self.quality = maxValue;
@@ -161,23 +168,23 @@ NSString * const HBVideoChangedNotification = @"HBVideoChangedNotification";
     [self postChangedNotification];
 }
 
-- (void)setTwoPass:(BOOL)twoPass
+- (void)setMultiPass:(BOOL)multiPass
 {
-    if (twoPass != _twoPass)
+    if (multiPass != _multiPass)
     {
-        [[self.undo prepareWithInvocationTarget:self] setTwoPass:_twoPass];
+        [[self.undo prepareWithInvocationTarget:self] setMultiPass:_multiPass];
     }
-    _twoPass = twoPass;
+    _multiPass = multiPass;
     [self postChangedNotification];
 }
 
-- (void)setTurboTwoPass:(BOOL)turboTwoPass
+- (void)setTurboMultiPass:(BOOL)turboMultiPass
 {
-    if (turboTwoPass != _turboTwoPass)
+    if (turboMultiPass != _turboMultiPass)
     {
-        [[self.undo prepareWithInvocationTarget:self] setTurboTwoPass:_turboTwoPass];
+        [[self.undo prepareWithInvocationTarget:self] setTurboMultiPass:_turboMultiPass];
     }
-    _turboTwoPass = turboTwoPass;
+    _turboMultiPass = turboMultiPass;
     [self postChangedNotification];
 }
 
@@ -282,14 +289,20 @@ NSString * const HBVideoChangedNotification = @"HBVideoChangedNotification";
 
 - (void)validateQualityType
 {
-    if (self.qualityType != 0 && !hb_video_quality_is_supported(self.encoder))
+    if (self.qualityType != HBVideoQualityTypeAvgBitrate && !hb_video_quality_is_supported(self.encoder))
     {
-        self.qualityType = 0;
+        self.qualityType = HBVideoQualityTypeAvgBitrate;
     }
 
-    if (!hb_video_twopass_is_supported(self.encoder))
+    if (self.qualityType != HBVideoQualityTypeConstantQuality && !hb_video_bitrate_is_supported(self.encoder))
     {
-        self.twoPass = NO;
+        self.qualityType = HBVideoQualityTypeConstantQuality;
+    }
+
+    if (! (hb_video_multipass_is_supported(self.encoder, false)
+            || hb_video_multipass_is_supported(self.encoder, true)))
+    {
+        self.multiPass = NO;
     }
 }
 
@@ -463,8 +476,8 @@ NSString * const HBVideoChangedNotification = @"HBVideoChangedNotification";
         copy->_frameRate = _frameRate;
         copy->_frameRateMode = _frameRateMode;
 
-        copy->_twoPass = _twoPass;
-        copy->_turboTwoPass = _turboTwoPass;
+        copy->_multiPass = _multiPass;
+        copy->_turboMultiPass = _turboMultiPass;
 
         copy->_preset = [_preset copy];
         copy->_tune = [_tune copy];
@@ -502,8 +515,8 @@ NSString * const HBVideoChangedNotification = @"HBVideoChangedNotification";
     encodeInt(_frameRate);
     encodeInteger(_frameRateMode);
 
-    encodeBool(_twoPass);
-    encodeBool(_turboTwoPass);
+    encodeBool(_multiPass);
+    encodeBool(_turboMultiPass);
 
     encodeObject(_preset);
     encodeObject(_tune);
@@ -530,8 +543,8 @@ NSString * const HBVideoChangedNotification = @"HBVideoChangedNotification";
 
     decodeInt(_frameRate); if (_frameRate < 0) { goto fail; }
     decodeInteger(_frameRateMode); if (_frameRateMode < HBVideoFrameRateModeVFR_PFR || _frameRateMode > HBVideoFrameRateModeCFR) { goto fail; }
-    decodeBool(_twoPass);
-    decodeBool(_turboTwoPass);
+    decodeBool(_multiPass);
+    decodeBool(_turboMultiPass);
 
     decodeObjectOrFail(_preset, NSString);
     decodeObjectOrFail(_tune, NSString);
@@ -689,10 +702,10 @@ fail:
     self.frameRate = intValue;
 
     // 2 Pass Encoding.
-    self.twoPass = [preset[@"VideoTwoPass"] boolValue];
+    self.multiPass = [preset[@"VideoMultiPass"] boolValue];
 
     // Turbo 1st pass for 2 Pass Encoding.
-    self.turboTwoPass = [preset[@"VideoTurboTwoPass"] boolValue];
+    self.turboMultiPass = [preset[@"VideoTurboMultiPass"] boolValue];
 
     self.notificationsEnabled = YES;
 }
@@ -748,8 +761,8 @@ fail:
         }
     }
 
-    preset[@"VideoTwoPass"] = @(self.twoPass);
-    preset[@"VideoTurboTwoPass"] = @(self.turboTwoPass);
+    preset[@"VideoMultiPass"] = @(self.multiPass);
+    preset[@"VideoTurboMultiPass"] = @(self.turboMultiPass);
 }
 
 @end
