@@ -10,6 +10,7 @@
 namespace HandBrake.Interop.Interop.Json.Scan
 {
     using System.Collections.Generic;
+    using System.Text.Json.Serialization;
 
     using HandBrake.Interop.Interop.Json.Shared;
 
@@ -81,7 +82,7 @@ namespace HandBrake.Interop.Interop.Json.Scan
         /// <summary>
         /// Gets or sets the meta data.
         /// </summary>
-        public SourceMetadata MetaData { get; set; }
+        public Dictionary<string, string> MetaData { get; set; }
 
         /// <summary>
         /// Gets or sets the name.
@@ -113,5 +114,12 @@ namespace HandBrake.Interop.Interop.Json.Scan
         /// Gets or sets the video codec.
         /// </summary>
         public string VideoCodec { get; set; }
-    }
+
+        public DVConfigRecord DolbyVisionConfigurationRecord { get; set; }
+
+        [JsonPropertyName("HDR10+")]
+        public int? HDR10plus { get; set; }
+
+        public MasteringDisplayColorVolume MasteringDisplayColorVolume { get; set; }
+    } 
 }
\ No newline at end of file
