@@ -10,6 +10,9 @@ namespace HandBrakeWPF.Model
     {
         None,
         Quality,
-        Bitrate
+        Bitrate,
+        Encoder,
+        Chapters,
+        Dimensions
     }
 }
