@@ -45,9 +45,12 @@ namespace HandBrakeWPF.Services.Encode.Factories
     {
         private readonly IUserSettingService userSettingService;
 
-        public EncodeTaskFactory(IUserSettingService userSettingService)
+        private readonly bool isEncodePath;
+
+        public EncodeTaskFactory(IUserSettingService userSettingService, bool isEncodePath)
         {
             this.userSettingService = userSettingService;
+            this.isEncodePath = isEncodePath;
         }
 
         internal JsonEncodeObject Create(EncodeTask job)
@@ -96,12 +99,27 @@ namespace HandBrakeWPF.Services.Encode.Factories
                     break;
             }
 
+            bool nvdec = this.userSettingService.GetUserSetting<bool>(UserSettingConstants.EnableNvDecSupport);
+
+            int hwDecode = 0;
+            if (nvdec)
+            {
+                hwDecode = (int)NativeConstants.HB_DECODE_SUPPORT_NVDEC;
+            }
+
+            bool qsv = this.userSettingService.GetUserSetting<bool>(UserSettingConstants.EnableQuickSyncDecoding);
+            if (qsv)
+            {
+                hwDecode |= (int)NativeConstants.HB_DECODE_SUPPORT_QSV;
+            }
+
             Source source = new Source
             {
                 Title = job.Title,
                 Range = range,
                 Angle = job.Angle,
                 Path = job.Source,
+                HWDecode = hwDecode
             };
             return source;
         }
@@ -111,10 +129,10 @@ namespace HandBrakeWPF.Services.Encode.Factories
             Destination destination = new Destination
             {
                 File = job.Destination,
-                Mp4Options = new Mp4Options
+                Options = new Options
                 {
                     IpodAtom = job.VideoEncoder.IsH264 ? job.IPod5GSupport : false,
-                    Mp4Optimize = job.OptimizeMP4
+                    Optimize = job.Optimize
                 },
                 ChapterMarkers = job.IncludeChapterMarkers,
                 AlignAVStart = job.AlignAVStart,
@@ -236,11 +254,12 @@ namespace HandBrakeWPF.Services.Encode.Factories
             if (job.VideoEncodeRateType == VideoEncodeRateType.AverageBitrate)
             {
                 video.Bitrate = job.VideoBitrate;
-                video.TwoPass = job.TwoPass;
-                video.Turbo = job.TurboFirstPass;
             }
 
+            video.MultiPass = job.MultiPass;
+            video.Turbo = job.TurboAnalysisPass;
 
+            bool enableQuickSyncEncoding = userSettingService.GetUserSetting<bool>(UserSettingConstants.EnableQuickSyncEncoding);
             bool enableQuickSyncDecoding = userSettingService.GetUserSetting<bool>(UserSettingConstants.EnableQuickSyncDecoding);
             bool useQSVDecodeForNonQSVEnc = userSettingService.GetUserSetting<bool>(UserSettingConstants.UseQSVDecodeForNonQSVEnc);
             bool enableQsvLowPower = userSettingService.GetUserSetting<bool>(UserSettingConstants.EnableQuickSyncLowPower);
@@ -251,14 +270,12 @@ namespace HandBrakeWPF.Services.Encode.Factories
             }
 
             // Allow use of the QSV decoder is configurable for non QSV encoders.
-            if (!job.VideoEncoder.IsHardwareEncoder && useQSVDecodeForNonQSVEnc)
+            if (this.isEncodePath &&  job.VideoEncoder != null && !job.VideoEncoder.IsHardwareEncoder && useQSVDecodeForNonQSVEnc && enableQuickSyncDecoding && enableQuickSyncEncoding)
             {
                 video.QSV.Decode = HandBrakeHardwareEncoderHelper.IsQsvAvailable && useQSVDecodeForNonQSVEnc;
             }
 
-            video.Options = job.ExtraAdvancedArguments;
-
-            if (HandBrakeHardwareEncoderHelper.IsQsvAvailable && (HandBrakeHardwareEncoderHelper.QsvHardwareGeneration > 6) && job.VideoEncoder.IsQuickSync)
+            if (!this.isEncodePath && HandBrakeHardwareEncoderHelper.IsQsvAvailable && (HandBrakeHardwareEncoderHelper.QsvHardwareGeneration > 6) && job.VideoEncoder.IsQuickSync)
             {
                 if (enableQsvLowPower && !video.Options.Contains("lowpower"))
                 {
@@ -270,11 +287,13 @@ namespace HandBrakeWPF.Services.Encode.Factories
                 }
             }
 
-            if (this.userSettingService.GetUserSetting<bool>(UserSettingConstants.EnableNvDecSupport) && job.VideoEncoder.IsNVEnc)
+            if (!this.isEncodePath && this.userSettingService.GetUserSetting<bool>(UserSettingConstants.EnableNvDecSupport) && job.VideoEncoder.IsNVEnc)
             {
                 video.HardwareDecode = (int)NativeConstants.HB_DECODE_SUPPORT_NVDEC;
             }
 
+            video.Options = job.ExtraAdvancedArguments;
+
             return video;
         }
 
@@ -586,27 +605,20 @@ namespace HandBrakeWPF.Services.Encode.Factories
             return filter;
         }
 
-        private Metadata CreateMetadata(EncodeTask job)
+        private Dictionary<string, string> CreateMetadata(EncodeTask job)
         {
-            if (job.MetaData != null && job.MetaData.PassthruMetadataEnabled)
+            if (job.MetaData != null && job.PassthruMetadataEnabled)
             {
-                Metadata metaData = new Metadata
-                                    {
-                                        Artist = job.MetaData.Artist,
-                                        Album = job.MetaData.Album,
-                                        AlbumArtist = job.MetaData.AlbumArtist,
-                                        Comment = job.MetaData.Comment,
-                                        Composer = job.MetaData.Composer,
-                                        Description = job.MetaData.Description,
-                                        Genre = job.MetaData.Genre,
-                                        LongDescription = job.MetaData.LongDescription,
-                                        Name = job.MetaData.Name,
-                                        ReleaseDate = job.MetaData.ReleaseDate
-                                    };
-                return metaData;
+                Dictionary<string, string> metadata = new Dictionary<string, string>();
+                foreach (var item in job.MetaData)
+                {
+                    metadata.Add(item.Annotation, item.Value);
+                }
+
+                return metadata;
             }
 
-            return new Metadata(); // Empty Metadata will not pass through to the destination.  
+            return new Dictionary<string, string>(); // Empty Metadata will not pass through to the destination.  
         }
     }
 }
