@@ -40,7 +40,7 @@ namespace HandBrakeWPF.Services.Encode.Model
             this.SubtitleTracks = new ObservableCollection<SubtitleTrack>();
             this.ChapterNames = new ObservableCollection<ChapterMarker>();
             this.AudioPassthruOptions = new ObservableCollection<HBAudioEncoder>();
-            this.MetaData = new MetaData();
+            this.MetaData = new ObservableCollection<MetaDataValue>();
             this.Padding = new PaddingFilter();
             this.VideoTunes = new List<VideoTune>();
         }
@@ -134,8 +134,8 @@ namespace HandBrakeWPF.Services.Encode.Model
             this.Quality = task.Quality;
             this.Framerate = task.Framerate;
             this.FramerateMode = task.FramerateMode;
-            this.TurboFirstPass = task.TurboFirstPass;
-            this.TwoPass = task.TwoPass;
+            this.TurboAnalysisPass = task.TurboAnalysisPass;
+            this.MultiPass = task.MultiPass;
             this.VideoBitrate = task.VideoBitrate;
             this.VideoEncoder = task.VideoEncoder;
             this.VideoEncodeRateType = task.VideoEncodeRateType;
@@ -148,11 +148,12 @@ namespace HandBrakeWPF.Services.Encode.Model
             /* Container */
             this.IPod5GSupport = task.IPod5GSupport;
             this.OutputFormat = task.OutputFormat;
-            this.OptimizeMP4 = task.OptimizeMP4;
+            this.Optimize = task.Optimize;
             this.AlignAVStart = task.AlignAVStart;
 
             /* Other */
-            this.MetaData = new MetaData(task.MetaData);
+            this.PassthruMetadataEnabled = task.PassthruMetadataEnabled;
+            this.MetaData = new ObservableCollection<MetaDataValue>(task.MetaData);
         }
 
         /* Source */
@@ -177,7 +178,7 @@ namespace HandBrakeWPF.Services.Encode.Model
 
         public OutputFormat OutputFormat { get; set; }
 
-        public bool OptimizeMP4 { get; set; }
+        public bool Optimize { get; set; }
 
         public bool IPod5GSupport { get; set; }
 
@@ -287,9 +288,9 @@ namespace HandBrakeWPF.Services.Encode.Model
 
         public int? VideoBitrate { get; set; }
 
-        public bool TwoPass { get; set; }
+        public bool MultiPass { get; set; }
 
-        public bool TurboFirstPass { get; set; }
+        public bool TurboAnalysisPass { get; set; }
 
         public double? Framerate { get; set; }
 
@@ -317,8 +318,8 @@ namespace HandBrakeWPF.Services.Encode.Model
 
 
         /* Metadata */
-        
-        public MetaData MetaData { get; set; }
+        public bool PassthruMetadataEnabled { get; set; }
+        public ObservableCollection<MetaDataValue> MetaData { get; set; }
 
         /* Previews */
 
