@@ -14,6 +14,7 @@ namespace HandBrakeWPF.Services.Queue
     using System.Collections.ObjectModel;
     using System.IO;
     using System.Linq;
+    using System.Linq.Expressions;
     using System.Text.Json;
     using System.Timers;
     using System.Windows;
@@ -48,6 +49,7 @@ namespace HandBrakeWPF.Services.Queue
         private static readonly object QueueLock = new object();
 
         private readonly List<ActiveJob> activeJobs = new List<ActiveJob>();
+        private readonly object activeJobLock = new object();
         private readonly IUserSettingService userSettingService;
         private readonly ILog logService;
         private readonly IErrorService errorService;
@@ -85,7 +87,7 @@ namespace HandBrakeWPF.Services.Queue
             this.allowedInstances = this.userSettingService.GetUserSetting<int>(UserSettingConstants.SimultaneousEncodes);
             this.processIsolationEnabled = this.userSettingService.GetUserSetting<bool>(UserSettingConstants.ProcessIsolationEnabled);
 
-            this.encodeTaskFactory = new EncodeTaskFactory(this.userSettingService);
+            this.encodeTaskFactory = new EncodeTaskFactory(this.userSettingService, true);
 
             this.hardwareResourceManager.Init();
         }
@@ -114,8 +116,17 @@ namespace HandBrakeWPF.Services.Queue
             }
         }
 
-        public int ActiveJobCount => this.activeJobs.Count;
-
+        public int ActiveJobCount
+        {
+            get
+            {
+                lock (activeJobLock)
+                {
+                    return this.activeJobs.Count;
+                }
+            }
+        }
+        
         public int ErrorCount
         {
             get
@@ -426,32 +437,46 @@ namespace HandBrakeWPF.Services.Queue
 
         public void Remove(QueueTask job)
         {
+            bool stoppedJob = false;
             lock (QueueLock)
             {
                 ActiveJob activeJob = null;
-                foreach (ActiveJob ajob in this.activeJobs)
+
+                lock (activeJobLock)
                 {
-                    if (Equals(ajob.Job, job))
+                    foreach (ActiveJob ajob in this.activeJobs)
                     {
-                        activeJob = ajob;
-                        ajob.Stop();
+                        if (Equals(ajob.Job, job))
+                        {
+                            activeJob = ajob;
+                            activeJob.JobFinished += (a, e) =>
+                            {
+                                this.activeJobs.Remove(activeJob);
+
+                                if (this.activeJobs.Count == 0 && this.IsPaused)
+                                {
+                                    this.IsPaused = false;
+                                }
+                            };
+                            ajob.Stop();
+                            stoppedJob = true;
+                            break;
+                        }
                     }
                 }
 
-                if (activeJob != null)
-                {
-                    this.activeJobs.Remove(activeJob);
-                }
-
                 this.queue.Remove(job);
-
-                if (this.activeJobs.Count == 0 && this.IsPaused)
-                {
-                    this.IsPaused = false;
-                }
             }
 
-            this.InvokeQueueChanged(EventArgs.Empty);
+            if (!stoppedJob)
+            {
+                this.InvokeQueueChanged(EventArgs.Empty);
+            }
+        }
+
+        private void ActiveJob_JobFinished1(object sender, ActiveJobCompletedEventArgs e)
+        {
+            throw new NotImplementedException();
         }
 
         public void RestoreQueue(string importPath)
@@ -509,15 +534,18 @@ namespace HandBrakeWPF.Services.Queue
         {
             if (pauseJobs)
             {
-                foreach (ActiveJob job in this.activeJobs)
+                lock (activeJobLock)
                 {
-                    if (job.IsEncoding && !job.IsPaused)
+                    foreach (ActiveJob job in this.activeJobs)
                     {
-                        job.Pause();
+                        if (job.IsEncoding && !job.IsPaused)
+                        {
+                            job.Pause();
+                        }
                     }
                 }
             }
-            
+
             this.IsProcessing = false;
             this.IsPaused = true;
 
@@ -535,16 +563,21 @@ namespace HandBrakeWPF.Services.Queue
 
             this.IsPaused = false;
 
+            ClearCancelledAndErrorJobs();
+
             this.allowedInstances = this.userSettingService.GetUserSetting<int>(UserSettingConstants.SimultaneousEncodes);
             this.processIsolationEnabled = this.userSettingService.GetUserSetting<bool>(UserSettingConstants.ProcessIsolationEnabled);
 
             this.IsProcessing = true;
 
             // Unpause all active jobs.
-            foreach (ActiveJob job in this.activeJobs)
+            lock (activeJobLock)
             {
-                job.Start();
-                this.InvokeJobProcessingStarted(new QueueProgressEventArgs(job.Job));
+                foreach (ActiveJob job in this.activeJobs)
+                {
+                    job.Start();
+                    this.InvokeJobProcessingStarted(new QueueProgressEventArgs(job.Job));
+                }
             }
 
             this.ProcessNextJob();
@@ -552,13 +585,27 @@ namespace HandBrakeWPF.Services.Queue
 
         public void Stop(bool stopExistingJobs)
         {
+            if (!this.IsProcessing)
+            {
+                // Just in Case, tidyup.
+                this.StopJobPolling();
+                this.RemoveBreakPoints();
+
+                this.IsPaused = false;
+
+                return;
+            }
+
             if (stopExistingJobs)
             {
-                foreach (ActiveJob job in this.activeJobs)
+                lock (activeJobLock)
                 {
-                    if (job.IsEncoding || job.IsPaused)
+                    foreach (ActiveJob job in this.activeJobs)
                     {
-                        job.Stop();
+                        if (job.IsEncoding || job.IsPaused)
+                        {
+                            job.Stop();
+                        }
                     }
                 }
 
@@ -568,9 +615,12 @@ namespace HandBrakeWPF.Services.Queue
                 this.StopJobPolling();
                 this.RemoveBreakPoints();
 
-                if (this.activeJobs.Count == 0)
+                lock (activeJobLock)
                 {
-                    this.InvokeQueueChanged(EventArgs.Empty);
+                    if (this.activeJobs.Count == 0)
+                    {
+                        this.InvokeQueueChanged(EventArgs.Empty);
+                    }
                 }
 
                 this.InvokeQueueCompleted(new QueueCompletedEventArgs(true));
@@ -586,23 +636,26 @@ namespace HandBrakeWPF.Services.Queue
 
         public List<QueueProgressStatus> GetQueueProgressStatus()
         {
-            // TODO make thread safe. 
             List<QueueProgressStatus> statuses = new List<QueueProgressStatus>();
-            foreach (ActiveJob job in this.activeJobs)
+            lock (activeJobLock)
             {
-                statuses.Add(job.Job.JobProgress);
+                foreach (ActiveJob job in this.activeJobs)
+                {
+                    statuses.Add(job.Job.JobProgress);
+                }
             }
-
             return statuses;
         }
 
         public List<string> GetActiveJobDestinationDirectories()
         {
-            // TODO need to make thread safe.
             List<string> directories = new List<string>();
-            foreach (ActiveJob job in this.activeJobs)
+            lock (activeJobLock)
             {
-                directories.Add(job.Job.Task.Destination);
+                foreach (ActiveJob job in this.activeJobs)
+                {
+                    directories.Add(job.Job.Task.Destination);
+                }
             }
 
             return directories;
@@ -624,8 +677,6 @@ namespace HandBrakeWPF.Services.Queue
                 {
                     this.queue.Insert(foundIndex, new QueueTask(QueueTaskType.Breakpoint));
                 }
-
-                this.IsProcessing = false;
             }
         }
 
@@ -680,9 +731,12 @@ namespace HandBrakeWPF.Services.Queue
                 this.allowedInstances = 1;
             }
 
-            if (this.activeJobs.Count >= this.allowedInstances)
+            lock (activeJobLock)
             {
-                return;
+                if (this.activeJobs.Count >= this.allowedInstances)
+                {
+                    return;
+                }
             }
 
             if (this.userSettingService.GetUserSetting<bool>(UserSettingConstants.ClearCompletedFromQueue))
@@ -715,14 +769,14 @@ namespace HandBrakeWPF.Services.Queue
                 ActiveJob activeJob = new ActiveJob(job, libEncode);
                 activeJob.JobFinished += this.ActiveJob_JobFinished;
                 activeJob.JobStatusUpdated += this.ActiveJob_JobStatusUpdated;
-                this.activeJobs.Add(activeJob);
+                lock (activeJobLock)
+                {
+                    this.activeJobs.Add(activeJob);
+                }
                 
                 activeJob.Start();
 
-                if (!this.QueueContainsStop())
-                {
-                    this.IsProcessing = true;
-                }
+                this.IsProcessing = true;
 
                 this.InvokeQueueChanged(EventArgs.Empty);
                 this.InvokeJobProcessingStarted(new QueueProgressEventArgs(job));
@@ -732,10 +786,19 @@ namespace HandBrakeWPF.Services.Queue
             {
                 this.BackupQueue(string.Empty);
 
-                if (!this.activeJobs.Any(a => a.IsEncoding))
+                bool queueComplete = false;
+                lock (activeJobLock)
                 {
-                    this.StopJobPolling();
+                    if (!this.activeJobs.Any(a => a.IsEncoding))
+                    {
+                        this.StopJobPolling();
 
+                        queueComplete = true;
+                    }
+                }
+
+                if (queueComplete)
+                {
                     // Fire the event to tell connected services.
                     this.InvokeQueueCompleted(new QueueCompletedEventArgs(false));
                 }
@@ -751,7 +814,11 @@ namespace HandBrakeWPF.Services.Queue
         {
             this.hardwareResourceManager.ReleaseToken(e.Job.Job.Task.VideoEncoder, e.Job.Job.TaskToken);
 
-            this.activeJobs.Remove(e.Job);
+            lock (activeJobLock)
+            {
+                this.activeJobs.Remove(e.Job);
+            }
+
             this.OnEncodeCompleted(e.EncodeEventArgs);
 
             this.InvokeQueueChanged(EventArgs.Empty);
@@ -759,9 +826,13 @@ namespace HandBrakeWPF.Services.Queue
 
         private void InvokeQueueCompleted(QueueCompletedEventArgs e)
         {
-            this.hardwareResourceManager.ClearTokens();
-            this.IsProcessing = false;
-            this.QueueCompleted?.Invoke(this, e);
+            ThreadHelper.OnUIThread(
+                () =>
+                {
+                    this.hardwareResourceManager.ClearTokens();
+                    this.IsProcessing = false;
+                    this.QueueCompleted?.Invoke(this, e);
+                });
         }
 
         private void OnQueueJobStatusChanged()
@@ -813,9 +884,12 @@ namespace HandBrakeWPF.Services.Queue
         {
             lock (QueueLock)
             {
-                if (this.activeJobs.Count != 0)
+                lock (activeJobLock)
                 {
-                    return; // Wait for jobs to finish!
+                    if (this.activeJobs.Count != 0)
+                    {
+                        return; // Wait for jobs to finish!
+                    }
                 }
 
                 this.StopJobPolling();
@@ -824,7 +898,6 @@ namespace HandBrakeWPF.Services.Queue
                 ThreadHelper.OnUIThread(() => this.queue.Remove(task));
             }
 
-
             this.IsProcessing = false;
             this.IsPaused = false;
 
@@ -845,5 +918,21 @@ namespace HandBrakeWPF.Services.Queue
         {
             return this.queue.Any(t => t.TaskType == QueueTaskType.Breakpoint);
         }
+
+        private void ClearCancelledAndErrorJobs()
+        {
+            // Called when queue starts to clear previous work off the queue that was left in an error state.
+            // This will remove confusion over the "Queue completed with errors" message at the end of the queue batch if the user doesn't clean it up themselves
+            lock (QueueLock)
+            {
+                foreach (QueueTask t in this.queue.ToList())
+                {
+                    if (t.Status == QueueItemStatus.Cancelled || t.Status == QueueItemStatus.Error)
+                    {
+                        this.queue.Remove(t);
+                    }
+                }
+            }
+        }
     }
 }
\ No newline at end of file
