@@ -10,7 +10,11 @@
 namespace HandBrakeWPF.Services.Scan.Factories
 {
     using System;
+    using System.Collections.Generic;
+    using System.IO;
 
+    using HandBrake.App.Core.Model;
+    using HandBrake.App.Core.Utilities;
     using HandBrake.Interop.Interop;
     using HandBrake.Interop.Interop.Interfaces.Model.Picture;
     using HandBrake.Interop.Interop.Json.Scan;
@@ -24,6 +28,22 @@ namespace HandBrakeWPF.Services.Scan.Factories
     {
         public Title CreateTitle(SourceTitle title, int mainFeature)
         {
+            string driveLabel = null;
+
+            foreach (DriveInformation info in DriveUtilities.GetDrives())
+            {
+                if (title.Path.StartsWith(info.RootDirectory, StringComparison.CurrentCultureIgnoreCase))
+                {
+                    driveLabel = info.VolumeLabel?.Trim();
+                    break;
+                }
+            }
+
+            if (string.IsNullOrEmpty(driveLabel))
+            {
+                driveLabel = Path.GetFileNameWithoutExtension(title.Path) ?? title.Path;
+            }
+
             Title converted = new Title
             {
                 TitleNumber = title.Index,
@@ -38,15 +58,60 @@ namespace HandBrakeWPF.Services.Scan.Factories
                     Left = title.Crop[2],
                     Right = title.Crop[3]
                 },
+                LooseCropDimensions = new Cropping()
+                {
+                    Top = title.LooseCrop[0],
+                    Bottom = title.LooseCrop[1],
+                    Left = title.LooseCrop[2],
+                    Right = title.LooseCrop[3]
+                },
                 Fps = ((double)title.FrameRate.Num) / title.FrameRate.Den,
-                SourceName = title.Path,
+                SourcePath = title.Path,
+                DriveLabel = driveLabel,
                 MainTitle = mainFeature == title.Index,
                 Playlist = title.Type == 1 ? string.Format(" {0:d5}.MPLS", title.Playlist).Trim() : null,
                 FramerateNumerator = title.FrameRate.Num,
                 FramerateDenominator = title.FrameRate.Den,
-                Type = title.Type
+                Type = title.Type,
+                ColorInformation = new ColorInfo
+                {
+                    HDR10plus = title.HDR10plus == 1,
+                    BitDepth = title.Color?.BitDepth,
+                    ChromaSubsampling = title.Color?.ChromaSubsampling,
+                }
             };
 
+            if (title.Color != null)
+            {
+                converted.ColorInformation.ColourInfoStr = string.Format(
+                    "{0}-{1}-{2}",
+                    title.Color.Primary,
+                    title.Color.Transfer,
+                    title.Color.Matrix);
+            }
+
+            if (title.MasteringDisplayColorVolume != null && title.MasteringDisplayColorVolume.HasPrimaries
+                                                          && title.MasteringDisplayColorVolume.HasLuminance)
+            {
+                converted.ColorInformation.HDR10 = true;
+            }
+
+            if (title.Color != null && (title.Color.Transfer == 16 || title.Color.Transfer == 18))
+            {
+                converted.ColorInformation.HDR = true;
+            }
+
+            if (title.DolbyVisionConfigurationRecord != null && title.DolbyVisionConfigurationRecord.DVProfile != null && converted.ColorInformation.HDR10plus)
+            {
+                converted.ColorInformation.DBV = true;
+                converted.ColorInformation.DBVProfileStr = string.Format("Dolby Vision {0}.{1} HDR10+", title.DolbyVisionConfigurationRecord.DVProfile, title.DolbyVisionConfigurationRecord.BLSignalCompatibilityId);
+            }
+            else if (title.DolbyVisionConfigurationRecord != null && title.DolbyVisionConfigurationRecord.DVProfile != null)
+            {
+                converted.ColorInformation.DBV = true;
+                converted.ColorInformation.DBVProfileStr = string.Format("Dolby Vision {0}.{1}", title.DolbyVisionConfigurationRecord.DVProfile, title.DolbyVisionConfigurationRecord.BLSignalCompatibilityId);
+            }
+
             int currentTrack = 1;
             foreach (SourceChapter chapter in title.ChapterList)
             {
@@ -74,20 +139,10 @@ namespace HandBrakeWPF.Services.Scan.Factories
                 currentSubtitleTrack++;
             }
 
-            SourceMetadata metadata = title.MetaData;
+            Dictionary<string, string> metadata = title.MetaData;
             if (title.MetaData != null)
             {
-                converted.Metadata = new Metadata(
-                    metadata.AlbumArtist,
-                    metadata.Album,
-                    metadata.Artist,
-                    metadata.Comment,
-                    metadata.Composer,
-                    metadata.Description,
-                    metadata.Genre,
-                    metadata.LongDescription,
-                    metadata.Name,
-                    metadata.ReleaseDate);
+                converted.Metadata = metadata;
             }
 
             return converted;
