@@ -15,6 +15,7 @@ namespace HandBrakeWPF.Services.Scan
     using System.Windows.Media.Imaging;
 
     using HandBrake.Interop.Interop;
+    using HandBrake.Interop.Interop.HbLib;
     using HandBrake.Interop.Interop.Interfaces;
     using HandBrake.Interop.Interop.Interfaces.Model.Preview;
     using HandBrake.Interop.Interop.Json.Encode;
@@ -42,8 +43,7 @@ namespace HandBrakeWPF.Services.Scan
         private readonly ILogInstanceManager logInstanceManager;
 
         private TitleFactory titleFactory = new TitleFactory();
-        private string currentSourceScanPath;
-        private IHandBrakeInstance instance;
+        private IScanInstance instance;
         private Action<bool, Source> postScanOperation;
         private bool isCancelled = false;
 
@@ -67,8 +67,8 @@ namespace HandBrakeWPF.Services.Scan
         /// Scan a Source Path.
         /// Title 0: scan all
         /// </summary>
-        /// <param name="sourcePath">
-        /// Path to the file to scan
+        /// <param name="sourcePaths">
+        /// Paths to the file to scan
         /// </param>
         /// <param name="title">
         /// int title number. 0 for scan all
@@ -76,7 +76,7 @@ namespace HandBrakeWPF.Services.Scan
         /// <param name="postAction">
         /// The post Action.
         /// </param>
-        public void Scan(string sourcePath, int title, Action<bool, Source> postAction)
+        public void Scan(List<string> sourcePaths, int title, Action<bool, Source> postAction)
         {
             if (this.IsScanning)
             {
@@ -110,7 +110,7 @@ namespace HandBrakeWPF.Services.Scan
             this.instance.ScanCompleted += this.InstanceScanCompleted;
 
             // Start the scan on a back
-            this.ScanSource(sourcePath, title, this.userSettingService.GetUserSetting<int>(UserSettingConstants.PreviewScanCount));
+            this.ScanSource(sourcePaths, title, this.userSettingService.GetUserSetting<int>(UserSettingConstants.PreviewScanCount));
         }
 
         /// <summary>
@@ -176,10 +176,13 @@ namespace HandBrakeWPF.Services.Scan
             BitmapImage bitmapImage = null;
             try
             {
-                EncodeTaskFactory factory = new EncodeTaskFactory(this.userSettingService);
+                EncodeTaskFactory factory = new EncodeTaskFactory(this.userSettingService, false);
                 JsonEncodeObject jobDict = factory.Create(job);
                 RawPreviewData bitmapData = this.instance.GetPreview(jobDict, preview);
-                bitmapImage = BitmapUtilities.ConvertToBitmapImage(BitmapUtilities.ConvertByteArrayToBitmap(bitmapData));
+                if (bitmapData != null)
+                {
+                    bitmapImage = BitmapUtilities.ConvertToBitmapImage(BitmapUtilities.ConvertByteArrayToBitmap(bitmapData));
+                }
             }
             catch (AccessViolationException e)
             {
@@ -203,7 +206,7 @@ namespace HandBrakeWPF.Services.Scan
         /// <summary>
         /// Start a scan for a given source path and title
         /// </summary>
-        /// <param name="sourcePath">
+        /// <param name="sourcePaths">
         /// Path to the source file
         /// </param>
         /// <param name="title">
@@ -212,22 +215,28 @@ namespace HandBrakeWPF.Services.Scan
         /// <param name="previewCount">
         /// The preview Count.
         /// </param>
-        private void ScanSource(object sourcePath, int title, int previewCount)
+        private void ScanSource(List<string> sourcePaths, int title, int previewCount)
         {
             try
             {
-                string source = sourcePath.ToString().EndsWith("\\") ? string.Format("\"{0}\\\\\"", sourcePath.ToString().TrimEnd('\\'))
-                              : "\"" + sourcePath + "\"";
-                this.currentSourceScanPath = source;
-
                 this.IsScanning = true;
 
                 TimeSpan minDuration = TimeSpan.FromSeconds(this.userSettingService.GetUserSetting<int>(UserSettingConstants.MinScanDuration));
 
                 HandBrakeUtils.SetDvdNav(!this.userSettingService.GetUserSetting<bool>(UserSettingConstants.DisableLibDvdNav));
 
+                List<string> excludedExtensions = this.userSettingService.GetUserSetting<List<string>>(UserSettingConstants.ExcludedExtensions);
+
+                bool nvdec = this.userSettingService.GetUserSetting<bool>(UserSettingConstants.EnableNvDecSupport);
+                int hwDecode = 0;
+                if (nvdec)
+                {
+                    hwDecode = (int)NativeConstants.HB_DECODE_SUPPORT_NVDEC;
+                }
+
+
                 this.ServiceLogMessage("Starting Scan ...");
-                this.instance.StartScan(sourcePath.ToString(), previewCount, minDuration, title != 0 ? title : 0);
+                this.instance.StartScan(sourcePaths, previewCount, minDuration, title != 0 ? title : 0, excludedExtensions, hwDecode);
 
                 this.ScanStarted?.Invoke(this, System.EventArgs.Empty);
             }
@@ -254,18 +263,11 @@ namespace HandBrakeWPF.Services.Scan
                 bool cancelled = this.isCancelled;
                 this.isCancelled = false;
 
-                // TODO -> Might be a better place to fix this.
-                string path = this.currentSourceScanPath;
-                if (this.currentSourceScanPath.Contains("\""))
-                {
-                    path = this.currentSourceScanPath.Trim('\"');
-                }
-
                 // Process into internal structures.
                 Source sourceData = null;
                 if (this.instance?.Titles != null)
                 {
-                    sourceData = new Source(path, this.ConvertTitles(this.instance.Titles), null);
+                    sourceData = new Source(this.ConvertTitles(this.instance.Titles));
                 }
 
                 this.IsScanning = false;
