@@ -31,11 +31,12 @@ namespace HandBrakeWPF.Services.Scan.Model
             this.AudioTracks = new List<Audio>();
             this.Chapters = new List<Chapter>();
             this.Subtitles = new List<Subtitle>();
-            this.Metadata = new Metadata();
+            this.Metadata = new Dictionary<string, string>();
+            this.ColorInformation = new ColorInfo();
         }
 
         #region Properties
-
+        
         /// <summary>
         /// Gets or sets a Collection of chapters in this Title
         /// </summary>
@@ -51,7 +52,7 @@ namespace HandBrakeWPF.Services.Scan.Model
         /// </summary>
         public List<Subtitle> Subtitles { get; set; }
 
-        public Metadata Metadata { get; set; }
+        public Dictionary<string, string> Metadata { get; set; }
 
         /// <summary>
         /// Gets or sets The track number of this Title
@@ -124,21 +125,35 @@ namespace HandBrakeWPF.Services.Scan.Model
         /// <summary>
         /// Gets or sets the Source Name
         /// </summary>
-        public string SourceName { get; set; }
+        public string SourcePath { get; set; }
 
         public string DisplaySourceName
         {
             get
             {
-                if (!string.IsNullOrEmpty(this.SourceName))
+                switch (this.Type)
                 {
-                    return Path.GetFileNameWithoutExtension(this.SourceName);
+                    case 0: // HB_DVD_TYPE
+                    case 1: // HB_BD_TYPE
+                 
+                        return DriveLabel;
+                    case 2: // HB_STREAM_TYPE
+                    case 3: // HB_FF_STREAM_TYPE
+                        if (!string.IsNullOrEmpty(this.SourcePath))
+                        {
+                            return Path.GetFileNameWithoutExtension(this.SourcePath);
+                        }
+                        break;
+                    default:
+                        return null;
                 }
 
                 return null;
             }
         }
 
+        public string DriveLabel { get; set; }
+
         public string SourceDisplayName
         {
             get
@@ -151,7 +166,7 @@ namespace HandBrakeWPF.Services.Scan.Model
                         return string.Empty;
                     case 2: // HB_STREAM_TYPE
                     case 3: // HB_FF_STREAM_TYPE
-                        return Path.GetFileNameWithoutExtension(this.SourceName);
+                        return Path.GetFileNameWithoutExtension(this.SourcePath);
                 }
             }
         }
@@ -185,6 +200,8 @@ namespace HandBrakeWPF.Services.Scan.Model
             }
         }
 
+        public ColorInfo ColorInformation { get; set; }
+
         #endregion
 
         /// <summary>
