@@ -16,25 +16,25 @@ namespace HandBrakeWPF.ViewModels
     using System.Diagnostics;
     using System.IO;
     using System.Linq;
-    using System.Runtime.Versioning;
-    using System.Threading;
-    using System.Threading.Tasks;
     using System.Windows;
+    using System.Windows.Input;
 
-    using Caliburn.Micro;
+    using HandBrake.App.Core.Exceptions;
+    using HandBrake.App.Core.Utilities;
+    using HandBrake.Interop.Interop;
 
+    using HandBrakeWPF.Commands;
+    using HandBrakeWPF.Commands.DebugTools;
     using HandBrakeWPF.EventArgs;
-    using HandBrakeWPF.Exceptions;
+    using HandBrakeWPF.Helpers;
     using HandBrakeWPF.Model.Options;
     using HandBrakeWPF.Properties;
     using HandBrakeWPF.Services.Interfaces;
     using HandBrakeWPF.Services.Queue.Interfaces;
     using HandBrakeWPF.Services.Queue.Model;
-    using HandBrakeWPF.Utilities;
+    using HandBrakeWPF.Utilities.FileDialogs;
     using HandBrakeWPF.ViewModels.Interfaces;
 
-    using Microsoft.Win32;
-
     public class QueueViewModel : ViewModelBase, IQueueViewModel
     {
         private readonly IErrorService errorService;
@@ -44,7 +44,8 @@ namespace HandBrakeWPF.ViewModels
         private WhenDone whenDoneAction;
         private QueueTask selectedTask;
         private bool isQueueRunning;
-
+        private bool extendedQueueDisplay;
+        
         public QueueViewModel(IUserSettingService userSettingService, IQueueService queueProcessor, IErrorService errorService)
         {
             this.userSettingService = userSettingService;
@@ -54,13 +55,27 @@ namespace HandBrakeWPF.ViewModels
             this.JobsPending = Resources.QueueViewModel_NoEncodesPending;
             this.SelectedItems = new BindingList<QueueTask>();
             this.SelectedItems.ListChanged += this.SelectedItems_ListChanged;
-            this.DisplayName = "Queue";
             this.IsQueueRunning = false;
             this.SelectedTabIndex = 0;
 
             this.WhenDoneAction = (WhenDone)this.userSettingService.GetUserSetting<int>(UserSettingConstants.WhenCompleteAction);
+            this.ExtendedQueueDisplay = this.userSettingService.GetUserSetting<bool>(UserSettingConstants.ExtendedQueueDisplay);
+
+            this.WhenDoneCommand = new SimpleRelayCommand<int>(this.WhenDone);
+            this.RetryCommand = new SimpleRelayCommand<QueueTask>(this.RetryJob);
+            this.EditCommand = new SimpleRelayCommand<QueueTask>(this.EditJob);
         }
 
+        public event EventHandler SimpleViewChanged;
+
+        public SimpleRelayCommand<QueueTask> EditCommand { get; set; }
+        public SimpleRelayCommand<QueueTask> RetryCommand { get; set; }
+        public SimpleRelayCommand<int> WhenDoneCommand { get; }
+
+        public ICommand QueueExtractResultDataCommand => new QueueExtractResultDataCommand(this, this.errorService);
+
+        public bool IsNightly => HandBrakeVersionHelper.IsNightly();
+
         public bool IsQueueRunning
         {
             get => this.isQueueRunning;
@@ -142,6 +157,8 @@ namespace HandBrakeWPF.ViewModels
 
         public bool CanPerformActionOnSource => this.SelectedTask != null;
 
+        public bool IsSimpleView { get; set; }
+
         public bool CanPlayFile =>
             this.SelectedTask != null && this.SelectedTask.Task != null && this.SelectedTask.Task.Destination != null && 
             this.SelectedTask.Status == QueueItemStatus.Completed && File.Exists(this.SelectedTask.Task.Destination);
@@ -151,7 +168,7 @@ namespace HandBrakeWPF.ViewModels
             get
             {
                 if (this.SelectedTask != null &&
-                    (this.selectedTask.Status == QueueItemStatus.Completed || this.selectedTask.Status == QueueItemStatus.Error || this.selectedTask.Status == QueueItemStatus.InProgress))
+                    (this.selectedTask.Status == QueueItemStatus.Completed || this.selectedTask.Status == QueueItemStatus.Error || this.selectedTask.Status == QueueItemStatus.Cancelled || this.selectedTask.Status == QueueItemStatus.InProgress))
                 {
                     return true;
                 }
@@ -160,6 +177,17 @@ namespace HandBrakeWPF.ViewModels
             }
         }
 
+        public bool ExtendedQueueDisplay
+        {
+            get => this.extendedQueueDisplay;
+            set
+            {
+                if (value == this.extendedQueueDisplay) return;
+                this.extendedQueueDisplay = value;
+                this.NotifyOfPropertyChange(() => this.ExtendedQueueDisplay);
+            }
+        }
+
         public void WhenDone(int action)
         {
             this.WhenDone(action, true);
@@ -174,7 +202,7 @@ namespace HandBrakeWPF.ViewModels
                 this.userSettingService.SetUserSetting(UserSettingConstants.WhenCompleteAction, action);
             }
 
-            IOptionsViewModel ovm = IoC.Get<IOptionsViewModel>();
+            IOptionsViewModel ovm = IoCHelper.Get<IOptionsViewModel>();
             ovm.UpdateSettings();
         }
 
@@ -195,7 +223,7 @@ namespace HandBrakeWPF.ViewModels
 
         public void Close()
         {
-            this.TryCloseAsync();
+            this.TryClose();
         }
 
         public override void OnLoad()
@@ -230,8 +258,8 @@ namespace HandBrakeWPF.ViewModels
             if (this.queueProcessor.IsEncoding)
             {
                 MessageBoxResult result = this.errorService.ShowMessageBox(
-                    "There are currently jobs running. Would you like to complete the current jobs before stopping the queue?",
-                    "Confirm",
+                    Resources.QueueViewModel_StopButContinueJob,
+                    Resources.Question,
                     MessageBoxButton.YesNoCancel,
                     MessageBoxImage.Question);
 
@@ -245,16 +273,17 @@ namespace HandBrakeWPF.ViewModels
                 }
                 else
                 {
+                    this.IsQueueRunning = false;
                     this.queueProcessor.Stop(true);
                 }
             }
             else
             {
+                this.IsQueueRunning = false;
                 this.queueProcessor.Stop(true);
             }
 
             this.JobsPending = string.Format(Resources.QueueViewModel_JobsPending, this.queueProcessor.Count);
-            this.IsQueueRunning = false;
         }
 
         public void RemoveSelectedJobs()
@@ -321,6 +350,16 @@ namespace HandBrakeWPF.ViewModels
             }
         }
 
+        public void RetrySelectedJob()
+        {
+            if (!CanRetryJob)
+            {
+                return;
+            }
+
+            this.RetryJob(this.SelectedTask);
+        }
+
         public void RetryJob(QueueTask task)
         {
             this.queueProcessor.RetryJob(task);
@@ -414,6 +453,16 @@ namespace HandBrakeWPF.ViewModels
             }
         }
 
+        public void EditSelectedJob()
+        {
+            if (!CanEditJob)
+            {
+                return;
+            }
+
+            this.EditJob(this.SelectedTask);
+        }
+
         public void EditJob(QueueTask task)
         {
             MessageBoxResult result = this.errorService.ShowMessageBox(
@@ -431,7 +480,7 @@ namespace HandBrakeWPF.ViewModels
             this.RemoveJob(task);
 
             // Pass a copy of the job back to the Main Screen
-            IMainViewModel mvm = IoC.Get<IMainViewModel>();
+            IMainViewModel mvm = IoCHelper.Get<IMainViewModel>();
             mvm.EditQueueJob(task);
         }
 
@@ -465,7 +514,7 @@ namespace HandBrakeWPF.ViewModels
         {
             foreach (var task in this.SelectedItems)
             {
-                if (task.Status == QueueItemStatus.Completed || task.Status == QueueItemStatus.Error)
+                if (task.Status == QueueItemStatus.Completed || task.Status == QueueItemStatus.Error || task.Status == QueueItemStatus.Cancelled)
                 {
                     this.RetryJob(task);
                 }
@@ -476,7 +525,7 @@ namespace HandBrakeWPF.ViewModels
         {
             foreach (var task in this.QueueTasks)
             {
-                if (task.Status == QueueItemStatus.Completed || task.Status == QueueItemStatus.Error)
+                if (task.Status == QueueItemStatus.Completed || task.Status == QueueItemStatus.Error || task.Status == QueueItemStatus.Cancelled)
                 {
                     this.RetryJob(task);
                 }
@@ -494,13 +543,25 @@ namespace HandBrakeWPF.ViewModels
             }
         }
 
+        public void ToggleSimpleQueueDisplay()
+        {
+            this.IsSimpleView = !this.IsSimpleView;
+            this.NotifyOfPropertyChange(() => this.IsSimpleView);
+            OnSimpleViewChanged();
+            this.userSettingService.SetUserSetting(UserSettingConstants.SimpleQueueView, this.IsSimpleView);
+        }
+
         public void PlayFile()
         {
             if (this.SelectedTask != null && this.SelectedTask.Task != null && File.Exists(this.SelectedTask.Task.Destination))
             {
                 try
                 {
-                    Process.Start(this.SelectedTask.Task.Destination);
+                    Process p = new()
+                                {
+                                    StartInfo = new(this.SelectedTask.Task.Destination) { UseShellExecute = true }
+                                };
+                    p.Start();
                 }
                 catch (Win32Exception exc)
                 {
@@ -521,17 +582,12 @@ namespace HandBrakeWPF.ViewModels
             this.queueProcessor.MoveToBottom(this.SelectedItems);
         }
 
-        public void Activate()
+        public void BackupQueue()
         {
-           this.OnActivateAsync(CancellationToken.None);
+            this.queueProcessor.BackupQueue(string.Empty);
         }
 
-        public void Deactivate()
-        {
-           this.OnDeactivateAsync(false, CancellationToken.None);
-        }
-
-        protected override Task OnActivateAsync(CancellationToken cancellationToken)
+        public override void Activate()
         {
             this.Load();
 
@@ -542,18 +598,22 @@ namespace HandBrakeWPF.ViewModels
 
             this.IsQueueRunning = this.queueProcessor.IsProcessing;
             this.JobsPending = string.Format(Resources.QueueViewModel_JobsPending, this.queueProcessor.Count);
-            
-            return base.OnActivateAsync(cancellationToken);
+
+            this.IsSimpleView = this.userSettingService.GetUserSetting<bool>(UserSettingConstants.SimpleQueueView);
+            this.NotifyOfPropertyChange(() => this.IsSimpleView);
+            this.OnSimpleViewChanged();
+
+            base.Activate();
         }
 
-        protected override Task OnDeactivateAsync(bool close, CancellationToken cancellationToken)
+        public override void Deactivate()
         {
             this.queueProcessor.QueueCompleted -= this.QueueProcessor_QueueCompleted;
             this.queueProcessor.QueueChanged -= this.QueueManager_QueueChanged;
             this.queueProcessor.JobProcessingStarted -= this.QueueProcessorJobProcessingStarted;
             this.queueProcessor.QueuePaused -= this.QueueProcessor_QueuePaused;
 
-            return base.OnDeactivateAsync(close, cancellationToken);
+            base.Deactivate();
         }
 
         private void OpenDirectory(string directory)
@@ -607,6 +667,12 @@ namespace HandBrakeWPF.ViewModels
             }
         }
 
+        public void ChangeQueueDisplay()
+        {
+            this.ExtendedQueueDisplay = !this.ExtendedQueueDisplay;
+            this.userSettingService.SetUserSetting(UserSettingConstants.ExtendedQueueDisplay, this.ExtendedQueueDisplay);
+        }
+
         private void HandleLogData()
         {
             if (this.SelectedTask == null || this.SelectedTask.Status == QueueItemStatus.InProgress || this.SelectedTask.Status == QueueItemStatus.Waiting)
@@ -621,10 +687,13 @@ namespace HandBrakeWPF.ViewModels
                     if (!string.IsNullOrEmpty(this.SelectedTask.Statistics.CompletedActivityLogPath)
                         && File.Exists(this.SelectedTask.Statistics.CompletedActivityLogPath))
                     {
-                        using (StreamReader logReader = new StreamReader(this.SelectedTask.Statistics.CompletedActivityLogPath))
+                        using (var fs = new FileStream(this.SelectedTask.Statistics.CompletedActivityLogPath, FileMode.Open, FileAccess.Read, FileShare.ReadWrite))
                         {
-                            string logContent = logReader.ReadToEnd();
-                            this.ActivityLog = logContent;
+                            using (StreamReader logReader = new StreamReader(fs))
+                            {
+                                string logContent = logReader.ReadToEnd();
+                                this.ActivityLog = logContent;
+                            }
                         }
                     }
                     else
@@ -692,5 +761,10 @@ namespace HandBrakeWPF.ViewModels
             this.JobsPending = string.Format(Resources.QueueViewModel_JobsPending, this.queueProcessor.Count);
             this.IsQueueRunning = false;
         }
+
+        protected virtual void OnSimpleViewChanged()
+        {
+            this.SimpleViewChanged?.Invoke(this, EventArgs.Empty);
+        }
     }
 }
\ No newline at end of file
